SystemOrganization addCategory: #'XPath-Tests'!

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:30'!
testHasPrefixedNamespaceURI
	| element namespaces |

	element := self newElementWithAttributes.
	namespaces := element namespaceScope prefixMappings values.
	element attributeNodesDo: [:each |
		each hasNamespaceURI
			ifTrue: [
				self assert:
					(each hasPrefixedNamespaceURI: each namespaceURI)]
			ifFalse: [
				self deny:
					(namespaces anySatisfy: [:namespace |
						each hasPrefixedNamespaceURI: namespace])]].! !

!XMLAttributeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass
			name: 'name'
			value: 'value') stringValue = 'value'! !

XMLNodeListTest subclass: #XPathNodeSetTest
	instanceVariableNames: 'document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathNodeSetTest methodsFor: 'testing' stamp: 'monty 1/18/2015 11:26'!
allowsDuplicates
	^ false! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 10/18/2015 07:57'!
assertList: aList copiedFrom: aSecondList equals: aThirdList
	super
		assertList: aList
		copiedFrom: aSecondList
		equals: aThirdList.
	self
		assert: aList set size = aList size;
		assert: aList set = aThirdList set;
		deny: aList set == aSecondList set. "internal set must be copied"! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 3/25/2015 22:50'!
assertList: aFirstList hasSameSortOrderAs: aSecondList
	self assert: aFirstList sortOrder class == aSecondList sortOrder class! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 4/9/2015 04:14'!
assertList: aList hasSortOrder: aSortOrder
	self assert: (aList sortOrder class == aSortOrder)! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 1/18/2015 09:35'!
assertList: aList isCopyOf: aSecondList
	super assertList: aList isCopyOf: aSecondList.
	self
		assert: aList set size = aList size;
		assert: aList set = aSecondList set;
		deny: aList set == aSecondList set.! !

!XPathNodeSetTest methodsFor: 'assertions' stamp: 'monty 10/18/2015 07:57'!
assertListHasUnknownSortOrder: aList
	self
		assertList: aList
		hasSortOrder: XPathUnknownSortOrder! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 1/18/2015 09:11'!
listClass
	^ XPathNodeSet! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 3/25/2015 22:15'!
listElements
	| listElements |

	listElements := super listElements.
	document
		ifNil: [
			(document := XMLDocument new)
				addElementNamed: 'root'.
			document root addNodes: listElements].
	^ listElements.
	! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 3/25/2015 22:42'!
listWithSortOrder
	^ self listWithElements sortOrder: XPathDocumentSortOrder new! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 6/7/2015 07:10'!
reverseSortBlock
	^ [:a :b | (self listElements indexOf: a) >= (self listElements indexOf: b)]! !

!XPathNodeSetTest methodsFor: 'accessing' stamp: 'monty 6/7/2015 06:36'!
sortBlock	
	^ [:a :b | (self listElements indexOf: a) <= (self listElements indexOf: b)]! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAdd
	super testAdd.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddAfter
	super testAddAfter.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement after: self singleElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddAfterIndex
	super testAddAfterIndex.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement afterIndex: 1;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddAll
	super testAddAll.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			addAll: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddAllFirst
	super testAddAllFirst.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			addAllFirst: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/17/2015 00:54'!
testAddAllIfNotPresentTo
	| list listToAdd |

	list := OrderedCollection new.
	listToAdd := self emptyList.
	self
		assert: (listToAdd addAllIfNotPresentTo: list) == listToAdd;
		assert: list isEmpty.

	listToAdd := self listWithElements.
	self
		assert: (listToAdd addAllIfNotPresentTo: list) == listToAdd;
		assert: list = self listElements asOrderedCollection;
		assert: (listToAdd addAllIfNotPresentTo: list) == listToAdd;
		assert: list = self listElements asOrderedCollection.
! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddAllLast
	super testAddAllLast.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			addAllLast: (Array with: self newElement);
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/17/2015 00:51'!
testAddAllTo
	| list listToAdd |

	list := OrderedCollection new.
	listToAdd := self emptyList.
	self
		assert: (listToAdd addAllTo: list) == listToAdd;
		assert: list isEmpty.

	listToAdd := self listWithElements.
	self
		assert: (listToAdd addAllTo: list) == listToAdd;
		assert: list = self listElements asOrderedCollection;
		assert: (listToAdd addAllTo: list) == listToAdd;
		assert: list = (self listElements, self listElements) asOrderedCollection.
! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddBefore
	super testAddBefore.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement before: self singleElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddBeforeIndex
	super testAddBeforeIndex.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			add: self newElement beforeIndex: 1;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddFirst
	super testAddFirst.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			addFirst: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAddLast
	super testAddLast.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			addLast: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:00'!
testAllButFirst
	| list |

	super testAllButFirst.

	list := self listWithSortOrder.
	self
		assertList: list allButFirst
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:01'!
testAllButFirstFew
	| list |

	super testAllButFirstFew.

	list := self listWithSortOrder.
	self
		assertList: (list allButFirst: 1)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:00'!
testAllButLast
	| list |

	super testAllButLast.

	list := self listWithSortOrder.
	self
		assertList: list allButLast
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:02'!
testAllButLastFew
	| list |

	super testAllButLastFew.

	list := self listWithSortOrder.
	self
		assertList: (list allButLast: 1)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:42'!
testAsXPathBoolean
	self
		deny: self emptyList asXPathBoolean;
		assert: self listWithSingleElement asXPathBoolean! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/29/2015 17:40'!
testAsXPathLocationPathRoot
	| list |

	list := self emptyList.
	self assert: list asXPathLocationPathRoot == list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/28/2015 16:30'!
testAsXPathNodeSet
	| list |

	list := self emptyList.
	self assert: list asXPathNodeSet == list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 4/7/2015 07:03'!
testAsXPathNumber
	self
		assert: self emptyList asXPathNumber isNaN;
		assert: self listWithElements asXPathNumber isNaN;
		assert: (XPathNodeSet with: ' 1.0 ' asXMLString) asXPathNumber = 1.0;
		assert: (XPathNodeSet with: ' 1.0 .' asXMLString) asXPathNumber isNaN! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:42'!
testAsXPathString
	self
		assert: self emptyList asXPathString isEmpty;
		assert: self listWithElements asXPathString = self singleElement stringValue;
		assert: self listWithElements reversed asXPathString = self singleElement stringValue! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testAtPut
	super testAtPut.

	self assertListHasUnknownSortOrder:
		(self listWithSortOrder
			at: 1 put: self newElement;
			yourself).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:49'!
testCopy
	| list |

	super testCopy.

	list := self listWithSortOrder.
	self assertList: list copy hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:53'!
testCopyAfter
	| list |

	super testCopyAfter.

	list := self listWithSortOrder.
	self
		assertList: (list copyAfter: list first)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:54'!
testCopyAfterLast
	| list |

	super testCopyAfterLast.

	list := self listWithSortOrder.
	self
		assertList: (list copyAfterLast: list first)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:32'!
testCopyEmpty
	| list |

	super testCopyEmpty.

	list := self listWithSortOrder.
	self
		assertList: list copyEmpty
		hasSameSortOrderAs: self emptyList.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyFromTo
	| list |

	super testCopyFromTo.

	list := self listWithSortOrder.
	self
		assertList: (list copyFrom: 1 to: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testCopyReplaceAllWith
	| list |

	super testCopyReplaceAllWith.

	list := self listWithSortOrder.
	self assertListHasUnknownSortOrder:
		(list
			copyReplaceAll: self listElements
			with: self listElements).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testCopyReplaceFromToWith
	| list |

	super testCopyReplaceFromToWith.

	list := self listWithSortOrder.
	self assertListHasUnknownSortOrder:
		(list
			copyReplaceFrom: 1
			to: list size
			with: self listElements).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyUpTo
	| list |

	super testCopyUpTo.

	list := self listWithSortOrder.
	self
		assertList: (list copyUpTo: list last)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:52'!
testCopyUpToLast
	| list |

	super testCopyUpToLast.

	list := self listWithSortOrder.
	self
		assertList: (list copyUpToLast: list last)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testCopyWith
	| list |

	super testCopyWith.

	list := self listWithSortOrder.
	self assertListHasUnknownSortOrder: (list copyWith: self newElement).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testCopyWithFirst
	| list |

	super testCopyWithFirst.

	list := self listWithSortOrder.
	self assertListHasUnknownSortOrder: (list copyWithFirst: self newElement).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:50'!
testCopyWithout
	| list |

	super testCopyWithout.

	list := self listWithSortOrder.
	self
		assertList: (list copyWithout: self singleElement)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:50'!
testCopyWithoutAll
	| list |

	super testCopyWithoutAll.

	list := self listWithSortOrder.
	self
		assertList: (list copyWithoutAll: (Array with: self singleElement))
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testDefaultSortOrder
	self
		assertList: self emptyList hasSortOrder: XPathDocumentSortOrder;
		assertList: self listWithSingleElement hasSortOrder: XPathDocumentSortOrder;
		assertListHasUnknownSortOrder: self listWithElements! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/27/2015 21:37'!
testDifference
	self assert:
		(self listWithElements \ self listWithElements) =
			self emptyList.
	self assert:
		(self listWithElements reversed \ self emptyList) =
			self listWithElements.
	self assert:
		(self listWithElements \ self listWithSingleElement) =
			self listWithElements allButFirst.
	self assert:
		(self listWithElements reversed \
			(self listElements select: [:each | each isElement])) =
				(self listWithElements reject: [:each | each isElement]).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:17'!
testFirstFew
	| list |

	super testFirstFew.

	list := self listWithSortOrder.
	self
		assertList: (list first: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 22:28'!
testFirstInDocumentOrder
	self
		assert: self emptyList firstInDocumentOrder isNil;
		assert: self listWithSingleElement firstInDocumentOrder = self singleElement;
		assert: self listWithElements firstInDocumentOrder = self listElements first;
		assert: self listWithElements reversed firstInDocumentOrder = self listElements first! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 4/7/2015 20:27'!
testIncludes
	| list element namespaceNode |

	list := self emptyList.
	self listElements do: [:each |
		self deny: (list includes: each).
		list add: each.
		self assert: (list includes: each)].

	self listElements do: [:each |
		list remove: each.
		self deny: (list includes: each)].

	element := XMLElement new.
	namespaceNode := XPathNamespaceNode withXMLNamespaceOnElement: element.
	self deny: (list includes: namespaceNode).
	list add: namespaceNode.
	self assert: (list includes: namespaceNode).

	"namespace nodes define ="
	self assert:
		(list includes:
			(XPathNamespaceNode withXMLNamespaceOnElement: element)).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/27/2015 21:37'!
testIntersection
	self assert:
		(self listWithElements reversed & self listWithElements) =
			self listWithElements.
	self assert:
		(self listWithElements & self emptyList) =
			self emptyList.
	self assert:
		(self listWithSingleElement & self listWithElements reversed) =
			self listWithSingleElement.
	self assert:
		(self listWithElements &
			(self listWithElements select: [:each | each isElement])) =
				(self listWithElements select: [:each | each isElement])! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:17'!
testLastFew
	| list |

	super testLastFew.

	list := self listWithSortOrder.
	self
		assertList: (list last: list size)
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:36'!
testReject
	| list |

	super testReject.

	list := self listWithSortOrder.
	self
		assertList: (list reject: [:each | false])
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:29'!
testRemoveAll
	| list |

	super testRemoveAll.

	(list := self listWithSortOrder) removeAll.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:29'!
testRemoveFirst
	| list |

	super testRemoveFirst.

	(list := self listWithSortOrder) removeFirst.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveFirstFew
	| list |

	super testRemoveFirstFew.

	(list := self listWithSortOrder) removeFirst: 1.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:59'!
testRemoveIfAbsent
	| list |

	super testRemoveIfAbsent.

	(list := self listWithSortOrder)
		remove: self absentElement ifAbsent: [nil];
		remove: self singleElement ifAbsent: [nil].
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveLast
	| list |

	super testRemoveLast.

	(list := self listWithSortOrder) removeLast.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:28'!
testRemoveLastFew
	| list |

	super testRemoveLastFew.

	(list := self listWithSortOrder) removeLast: 1.
	self
		assertList: list
		hasSameSortOrderAs: self listWithSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testReplaceFromToWith
	| list |

	super testReplaceFromToWith.

	(list := self listWithSortOrder)
		replaceFrom: 1
		to: list size
		with: self listElements.
	self assertListHasUnknownSortOrder: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testReplaceFromToWithStartingAt
	| list |

	super testReplaceFromToWithStartingAt.

	(list := self listWithSortOrder)
		replaceFrom: 1
		to: list size
		with: self listElements
		startingAt: 1.
	self assertListHasUnknownSortOrder: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 08:02'!
testReverse
	| list copy |

	super testReverse.

	list := self listWithElements.
	copy := list reverse.
	self
		assertListHasUnknownSortOrder: copy;
		assertList: copy
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reverse).

	list := self listWithSortOrder.
	copy := list reverse.
	self
		assertList: copy hasSortOrder: XPathReverseDocumentSortOrder;
		assertList: copy
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reverse).

	list := copy.
	copy := list reverse.
	self
		assertList: copy hasSortOrder: XPathDocumentSortOrder;
		assertList: copy
		copiedFrom: list
		equals: self listWithElements.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 09:47'!
testReverseInPlace
	| list |

	list := self emptyList.
	self
		assert: list reverseInPlace == list;
		assert: list isEmpty.

	list := self listWithSingleElement.
	self
		assert: list reverseInPlace == list;
		assert: list = self listWithSingleElement;
		assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

	list := self listWithElements.
	self
		assert: list reverseInPlace == list;
		assert: list = (self listClass newFrom: self listElements reversed);
		assertListHasUnknownSortOrder: list.

	list := self listWithSortOrder.
	self
		assert: list reverseInPlace == list;
		assert: list = (self listClass newFrom: self listElements reversed);
		assertList: list hasSortOrder: XPathReverseDocumentSortOrder;
		assert: list reverseInPlace == list;
		assert: list = (self listClass newFrom: self listElements);
		assertList: list hasSortOrder: XPathDocumentSortOrder.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 08:01'!
testReversed
	| list copy |

	super testReversed.

	list := self listWithElements.
	copy := list reversed.
	self
		assertListHasUnknownSortOrder: copy;
		assertList: copy
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reversed).

	list := self listWithSortOrder.
	copy := list reversed.
	self
		assertList: copy hasSortOrder: XPathReverseDocumentSortOrder;
		assertList: copy
		copiedFrom: list
		equals: (self listClass newFrom: self listElements reversed).

	list := copy.
	copy := list reversed.
	self
		assertList: copy hasSortOrder: XPathDocumentSortOrder;
		assertList: copy
		copiedFrom: list
		equals: self listWithElements.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/25/2015 23:36'!
testSelect
	| list |

	super testSelect.

	list := self listWithSortOrder.
	self
		assertList: (list select: [:each | true])
		hasSameSortOrderAs: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testSort
	super testSort.

	self
		assertList: self listWithElements sort
		hasSortOrder: XPathDocumentSortOrder.
	self
		should: [(self listWithElements copyWith: self newElement) sort]
		raise: XPathDOMException.
	self
		shouldnt: [self emptyList sort]
		raise: Error.

	self assertListHasUnknownSortOrder:
		(self listWithElements
			sort;
			sort: self sortBlock).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/7/2015 03:53'!
testSorted
	super testSorted.

	self
		assertList: self listWithElements sorted
		hasSortOrder: XPathDocumentSortOrder.
	self
		should: [(self listWithElements copyWith: self newElement) sorted]
		raise: XPathDOMException.
	self
		shouldnt: [self emptyList sorted]
		raise: Error.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 8/16/2015 21:30'!
testStringValue
	| firstStringValue |

	self assert: self emptyList stringValue = ''.

	firstStringValue := self listWithElements stringValues first.
	self
		assert: self listWithElements stringValue = firstStringValue;
		assert: (self listWithElements last <<< 'node()') stringValue = firstStringValue.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/27/2015 01:12'!
testStringValues
	self
		assert: self emptyList stringValues isEmpty;
		assert:
			self listWithElements stringValues =
				(self listWithElements collect: [:each | each stringValue])! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/27/2015 01:13'!
testStringValuesDo
	| list |

	list := self emptyList.
	self
		should: [:block | list stringValuesDo: block]
		enumerate: #().

	list := self listWithElements.
	self
		should: [:block | list stringValuesDo: block]
		enumerate: (self listWithElements collect: [:each | each stringValue]).! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 10/18/2015 07:57'!
testSwapWith
	| list |

	super testSwapWith.

	(list := self listWithSortOrder)
		swap: 1
		with: list size.
	self assertListHasUnknownSortOrder: list.! !

!XPathNodeSetTest methodsFor: 'tests' stamp: 'monty 3/27/2015 21:28'!
testUnion
	self assert:
		(self listWithElements | self listWithElements reversed) =
			self listWithElements.
	self assert:
		(self listWithElements | self emptyList) =
			self listWithElements.
	self assert:
		(self listWithSingleElement | self listWithElements reversed) =
			self listWithElements.
	self assert:
		((self listWithElements select: [:each | each isElement]) reversed |
			(self listWithElements reject: [:each | each isElement])) =
				self listWithElements.! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 4/3/2015 07:50'!
testAllNodesReverseDo
	| nodeWithNodes |

	super testAllNodesReverseDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each allNodesReverseDo: block]
			enumerate:
				(each hasChildren
					ifTrue: [each allNodes reversed]
					ifFalse: [Array with: each])].! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 11:24'!
testDocumentRoot
	| nodeWithNodes |

	super testDocumentRoot.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self assert: each documentRoot == nodeWithNodes].! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:59'!
testFollowingNodesDo
	| nodeWithNodes |

	super testFollowingNodesDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each followingNodesDo: block]
			enumerate:
				(each hasParent
					ifTrue: [
						each parent descendentNodes copyAfter:
							(each hasChildren
								ifTrue: [each descendentNodes last]
								ifFalse: [each])]
					ifFalse: [#()])].! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:55'!
testFollowingSiblingNodesDo
	| nodeWithNodes |

	super testFollowingSiblingNodesDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each followingSiblingNodesDo: block]
			enumerate:
				(each hasParent
					ifTrue: [each parent nodes copyAfter: each]
					ifFalse: [#()])].! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:56'!
testPrecedingNodesDo
	| nodeWithNodes |

	super testPrecedingNodesDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each precedingNodesDo: block]
			enumerate:
				(each hasParent
					ifTrue: [(each parent descendentNodes copyUpTo: each) reversed]
					ifFalse: [#()])].! !

!XMLNodeWithChildrenTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:57'!
testPrecedingSiblingNodesDo
	| nodeWithNodes |

	super testPrecedingSiblingNodesDo.

	nodeWithNodes := self newNodeWithNodes.
	nodeWithNodes allNodesDo: [:each |
		self
			should: [:block | each precedingSiblingNodesDo: block]
			enumerate:
				(each hasParent
					ifTrue: [(each parent nodes copyUpTo: each) reversed]
					ifFalse: [#()])].! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:18'!
testExpandedName
	self assert: (self nodeClass target: 'target' data: 'data') expandedName = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:15'!
testHasNamespaceURI
	self deny: (self nodeClass target: 'target' data: 'data') hasNamespaceURI! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:16'!
testHasPrefix
	self deny: (self nodeClass target: 'target' data: 'data') hasPrefix! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:37'!
testHasPrefixedNamespaceURI
	| pi |

	pi := self nodeClass target: 'target' data: 'data'.
	self
		deny: (pi hasPrefixedNamespaceURI: 'target');
		deny: (pi hasPrefixedNamespaceURI: 'data');
		deny: (pi hasPrefixedNamespaceURI: '').! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:14'!
testLocalName
	self assert: (self nodeClass target: 'target' data: 'data') localName = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:14'!
testName
	self assert: (self nodeClass target: 'target' data: 'data') name = 'target'! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:15'!
testNamespaceURI
	self assert: (self nodeClass target: 'target' data: 'data') namespaceURI isEmpty! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 14:07'!
testPrefix
	self assert: (self nodeClass target: 'target' data: 'data') prefix isEmpty! !

!XMLPITest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass
			target: 'target'
			data: 'data') stringValue = 'data'.! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:14'!
assertList: aList hasSortOrder: aSortOrder
	self assert: (aList sortOrder class == aSortOrder)! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/27/2015 01:25'!
testAllNodesReverseDo
	| node |

	node := self nodeClass new.
	self
		should: [:block | node allNodesReverseDo: block]
		enumerate: (Array with: node).! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:33'!
testAncestorAxis
	self
		assert: (self nodeClass new %% 'node()') isEmpty;
		assert: (self nodeClass new %% '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 6/30/2015 06:39'!
testAncestorOrSelfAxis
	| node list |

	node := self nodeClass new.
	list := node %%~ 'node()'.
	self
		assert: list = (XPathNodeSet with: node);
		assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

	list := node %%~ '*'.
	node isElement
		ifTrue: [
			self
				assert: list = (XPathNodeSet with: node);
				assertList: list hasSortOrder: XPathReverseDocumentSortOrder]
		ifFalse: [self assert: list isEmpty].! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 10/29/2015 17:08'!
testAsXPathLocationPathRoot
	| node |

	node := self nodeClass new.
	self assert: node asXPathLocationPathRoot == node.! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 6/30/2015 19:06'!
testAttributeAxis
	self
		assert: (self nodeClass new @ 'node()') isEmpty;
		assert: (self nodeClass new @ '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:32'!
testChildAxis
	self
		assert: (self nodeClass new / 'node()') isEmpty;
		assert: (self nodeClass new / '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:33'!
testDescendantAxis
	self
		assert: (self nodeClass new // 'node()') isEmpty;
		assert: (self nodeClass new // '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 10/7/2015 03:51'!
testDescendantOrSelfAxis
	| node list |

	node := self nodeClass new.
	list := node //~ 'node()'.
	self
		assert: list = (XPathNodeSet with: node);
		assertList: list hasSortOrder: XPathDocumentSortOrder.

	list := node //~ '*'.
	node isElement
		ifTrue: [
			self
				assert: list = (XPathNodeSet with: node);
				assertList: list hasSortOrder: XPathDocumentSortOrder]
		ifFalse: [self assert: list isEmpty].! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 11:06'!
testDocumentRoot
	| node |

	node := self nodeClass new.
	self assert: node documentRoot == node.! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 05:01'!
testFollowingAxis
	self
		assert: (self nodeClass new >> 'node()') isEmpty;
		assert: (self nodeClass new >> '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:29'!
testFollowingNodesDo
	| node |

	node := self nodeClass new.
	self
		should: [:block | node followingNodesDo: block]
		enumerate: #().! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 05:01'!
testFollowingSiblingAxis
	self
		assert: (self nodeClass new >>> 'node()') isEmpty;
		assert: (self nodeClass new >>> '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:42'!
testFollowingSiblingNodesDo
	| node |

	node := self nodeClass new.
	self
		should: [:block | node followingSiblingNodesDo: block]
		enumerate: #().! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:56'!
testHasExpandedName
	self deny: self nodeClass new hasExpandedName! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:02'!
testIsNamespace
	self deny: self nodeClass new isNamespace! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 6/30/2015 19:12'!
testNamespaceAxis
	self
		assert: (self nodeClass new @@ 'node()') isEmpty;
		assert: (self nodeClass new @@ '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:36'!
testParentAxis
	self
		assert: (self nodeClass new % 'node()') isEmpty;
		assert: (self nodeClass new % '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:36'!
testPrecedingAxis
	self
		assert: (self nodeClass new << 'node()') isEmpty;
		assert: (self nodeClass new << '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:29'!
testPrecedingNodesDo
	| node |

	node := self nodeClass new.
	self
		should: [:block | node precedingNodesDo: block]
		enumerate: #().! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 05:00'!
testPrecedingSiblingAxis
	self
		assert: (self nodeClass new <<< 'node()') isEmpty;
		assert: (self nodeClass new <<< '*') isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/1/2015 23:42'!
testPrecedingSiblingNodesDo
	| node |

	node := self nodeClass new.
	self
		should: [:block | node precedingSiblingNodesDo: block]
		enumerate: #().! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 10/7/2015 03:51'!
testSelfAxis
	| node list |

	node := self nodeClass new.
	list := node ~ 'node()'.
	self
		assert: list = (XPathNodeSet with: node);
		assertList: list hasSortOrder: XPathDocumentSortOrder.

	list := node ~ '*'.
	node isElement
		ifTrue: [
			self
				assert: list = (XPathNodeSet with: node);
				assertList: list hasSortOrder: XPathDocumentSortOrder]
		ifFalse: [self assert: list isEmpty].! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 10:56'!
testStringValue
	self assert: self nodeClass new stringValue isEmpty! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/7/2015 18:39'!
testXPath
	| node |

	node := self nodeClass new.
	self
		assert: (node xPath: '.') = (XPathNodeSet with: node);
		assert: (node xPath: (XPath for: '.')) = (XPathNodeSet with: node).! !

!XMLNodeTest methodsFor: '*XPath-Tests' stamp: 'monty 4/7/2015 18:41'!
testXPathContext
	| node context |

	node := self nodeClass new.
	(context := XPathContext new)
		variableAt: 'var' put: 'test'.
	self
		assert: (node xPath: '$var' context: context) = 'test';
		assert: (node xPath: (XPath for: '$var') context: context) = 'test'.! !

XMLNodeTest subclass: #XPathNamespaceNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathNamespaceNodeTest methodsFor: 'instance creation' stamp: 'monty 3/26/2015 23:21'!
newElementWithNamespaceNode
	^ self newElementWithAttributes
		declarePrefix: 'prefix'
		uri: 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'instance creation' stamp: 'monty 3/10/2015 13:50'!
newNamespaceNode
	^ self nodeClass
		element: self newElementWithAttributes
		name: 'prefix'
		uri: 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'accessing' stamp: 'monty 2/13/2015 08:40'!
nodeClass
	^ XPathNamespaceNode! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:53'!
testEquals
	| firstNamespace secondNamespace |

	firstNamespace := self newNamespaceNode.
	self assert: firstNamespace = firstNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: firstNamespace name
			uri: firstNamespace uri.
	self assert: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: self newElementWithAttributes
			name: firstNamespace name
			uri: firstNamespace uri.
	self deny: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: 'newPrefix'
			uri: firstNamespace uri.
	self deny: firstNamespace = secondNamespace.

	secondNamespace :=
		self nodeClass
			element: firstNamespace element
			name: firstNamespace name
			uri: 'newURI'.
	self deny: firstNamespace = secondNamespace.! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testExpandedName
	self assert: self newNamespaceNode expandedName = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 12:58'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:08'!
testHasNamespaceURI
	self deny: self newNamespaceNode hasNamespaceURI! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:45'!
testHasPrefix
	self deny: self newNamespaceNode hasPrefix! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:16'!
testHasPrefixedNamespaceURI
	| namespace |

	namespace := self newNamespaceNode.
	self deny: (namespace hasPrefixedNamespaceURI: namespace uri).! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 13:02'!
testIsNamespace
	self assert: self nodeClass new isNamespace! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testLocalName
	self assert: self newNamespaceNode localName = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testName
	self assert: self newNamespaceNode name = 'prefix'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:09'!
testNamespaceURI
	self assert: self newNamespaceNode namespaceURI isEmpty! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 11:05'!
testPrefix
	"namespace nodes model prefixes, but cannot be prefixed themselves"
	self assert: self newNamespaceNode prefix isEmpty! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert: self newNamespaceNode stringValue = 'prefixNS'.! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:16'!
testURI
	self assert: self newNamespaceNode uri = 'prefixNS'! !

!XPathNamespaceNodeTest methodsFor: 'tests' stamp: 'monty 3/10/2015 14:28'!
testWriteXMLOn
	self assert: self newNamespaceNode printString = 'xmlns:prefix="prefixNS"'.
	self assert:
		(self nodeClass withXMLNamespaceOnElement: nil) printString =
			('xmlns:xml="', XMLNamespaceScope xmlNamespaceURI, '"').
	self assert:
		(self nodeClass
			element: nil
			name: ''
			uri: 'defaultNS') printString = 'xmlns="defaultNS"'.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 10/8/2015 10:46'!
testAttributeAxis
	| element list |

	super testAttributeAxis.

	element := self newElementWithAttributes.
	list := element @ '*'.
	self
		assert: list = (XPathNodeSet newFrom: element attributeNodes);
		assertList: list hasSortOrder: XPathDocumentSortOrder.

	element := self nodeClass new.
	self assert: (element @ 'node()') isEmpty.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 12:57'!
testHasExpandedName
	self assert: self nodeClass new hasExpandedName! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:40'!
testHasNamespaceNodes
	| element |

	element := self nodeClass new.
	self assert: element hasNamespaceNodes.

	element declareNamespaces: self namespaces.
	self assert: element hasNamespaceNodes.

	element configuration usesNamespaces: false.
	self deny: element hasNamespaceNodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/10/2015 13:35'!
testHasPrefixedNamespaceURI
	| element |

	element := self nodeClass name: 'test' namespaces: self namespaces.
	self
		assert: element namespaceURI notEmpty;
		assert: element prefix isEmpty;
		deny: (element hasPrefixedNamespaceURI: '').

	element name: 'prefix:test'.
	self assert: (element hasPrefixedNamespaceURI: 'prefixNS').! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 10/6/2015 21:22'!
testNamespaceAxis
	| element list |

	element := self nodeClass new.
	element declareNamespaces: self namespaces.
	list := element @@ '*'.
	self
		assert: list = (XPathNodeSet newFrom: element namespaceNodes);
		assertList: list hasSortOrder: XPathDocumentSortOrder.

	element := self nodeClass new.
	element configuration usesNamespaces: false.
	self assert: (element @@ 'node()') isEmpty.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 23:39'!
testNamespaceNodes
	| element nodes |

	element := self nodeClass new.
	nodes :=
		XMLNodeList with:
			(XPathNamespaceNode withXMLNamespaceOnElement: element).
	self assert: element namespaceNodes = nodes.

	element configuration usesNamespaces: false.
	self assert: element namespaceNodes isEmpty.

	element configuration usesNamespaces: true.
	element declareNamespaces: self namespaces.
	self namespaces keysAndValuesDo: [:prefix :uri |
		nodes addLast:
			(XPathNamespaceNode
				element: element
				name: prefix
				uri: uri)].
	self assert: element namespaceNodes = nodes.

	element declarePrefix: 'xml' uri: XMLNamespaceScope xmlNamespaceURI.
	nodes addLast: nodes removeFirst.
	self assert: element namespaceNodes = nodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 23:39'!
testNamespaceNodesDo
	| element |

	element := self nodeClass new.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.

	element configuration usesNamespaces: false.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: #().

	element configuration usesNamespaces: true.
	element declareNamespaces: self namespaces.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.

	element declarePrefix: 'xml' uri: XMLNamespaceScope xmlNamespaceURI.
	self
		should: [:block | element namespaceNodesDo: block]
		enumerate: element namespaceNodes.! !

!XMLElementTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass new
			addContent: 'before';
			addNode:
				((self nodeClass named: 'element')
					addContent: 'content';
					yourself);
			addContent: 'after';
			yourself) stringValue = 'beforecontentafter'.! !

!XMLStringTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:14'!
testStringValue
	super testStringValue.

	self assert:
		(self nodeClass string: self markupString) stringValue =
			self markupString.! !

TestCase subclass: #XPathFunctionSetTest
	instanceVariableNames: 'functions disjointFunctions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:26'!
absentFunction
	^ XPathStringFunction! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/19/2015 16:22'!
assertSet: aFirstSet equals: aSecondSet
	self
		assert: aFirstSet = aSecondSet;
		assert: aSecondSet = aFirstSet! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/19/2015 16:23'!
denySet: aFirstSet equals: aSecondSet
	self
		deny: aFirstSet = aSecondSet;
		deny: aFirstSet == aSecondSet;
		deny: aSecondSet = aFirstSet;
		deny: aSecondSet == aFirstSet! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:32'!
disjointFunctions
	disjointFunctions ifNil: [
		(disjointFunctions := OrderedCollection new)
			addLast: XPathSubstringFunction;
			addLast: XPathSubstringBeforeFunction;
			addLast: XPathSubstringAfterFunction.
		disjointFunctions := disjointFunctions asArray].
	^ disjointFunctions copy.! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
emptySet
	^ self setClass new! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:26'!
functions
	functions ifNil: [
		(functions := OrderedCollection new)
			addLast: XPathTrueFunction;
			addLast: XPathFalseFunction;
			addLast: XPathNotFunction.
		functions := functions asArray].
	^ functions copy.! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
setClass
	^ XPathFunctionSet! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:39'!
setWithDisjointFunctions
	^ self setClass newFrom: self disjointFunctions! !

!XPathFunctionSetTest methodsFor: 'accessing' stamp: 'monty 1/19/2015 15:15'!
setWithFunctions
	^ self setClass newFrom: self functions! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/20/2015 14:38'!
should: aBlock add: aFirstObject and: aSecondObject equaling: anExpectedSum
	| firstCopy secondCopy sum |

	firstCopy := aFirstObject asFunctionSet copy.
	secondCopy := aSecondObject asFunctionSet copy.
	sum := aBlock value.
	self
		assert: sum = anExpectedSum;
		deny: sum == aFirstObject;
		deny: sum == aSecondObject;
		assert: firstCopy = aFirstObject asFunctionSet;
		assert: secondCopy = aSecondObject asFunctionSet.! !

!XPathFunctionSetTest methodsFor: 'asserting' stamp: 'monty 1/20/2015 14:47'!
should: aBlock subtract: aFirstObject from: aSecondObject equaling: anExpectedDifference
	| firstCopy secondCopy difference |

	firstCopy := aFirstObject asFunctionSet copy.
	secondCopy := aSecondObject asFunctionSet copy.
	difference := aBlock value.
	self
		assert: difference = anExpectedDifference;
		deny: difference == aFirstObject;
		deny: difference == aSecondObject;
		assert: firstCopy = aFirstObject asFunctionSet;
		assert: secondCopy = aSecondObject asFunctionSet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 15:19'!
testAdd
	| set |

	set := self emptySet.
	self functions do: [:each |
		self
			assert: (set add: each) == each;
			assert: (set includes: each)].
	self
		assert: set = self setWithFunctions;
		assert: (self functions allSatisfy: [:each | (set add: each) == each]);
		assert: set = self setWithFunctions.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:00'!
testAsFunctionSet
	| set function |

	set := self setClass new.
	self assert: set asFunctionSet == set.

	function := self functions anyOne.
	set := function asFunctionSet.
	self assert: set = (self setClass with: function).
	! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 4/3/2015 07:47'!
testConcatenation
	| firstSet secondSet |

	firstSet := self setWithFunctions.
	self assert: (firstSet, self setWithFunctions) = self setWithFunctions.

	secondSet := self setWithDisjointFunctions.	
	self
		assert: (firstSet, secondSet) = (secondSet, firstSet);
		deny: (firstSet, secondSet) = firstSet;
		deny: (firstSet, secondSet) = secondSet;
		assert:
			(firstSet, secondSet) =
				(self setClass newFrom: self functions, self disjointFunctions).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:45'!
testDo
	| enumerated |

	enumerated := OrderedCollection new.
	self emptySet do: [:each | enumerated addLast: each].
	self assert: enumerated size = 0.

	self setWithFunctions do: [:each | enumerated addLast: each].
	self
		assert: enumerated size = self functions size;
		assert: (self functions allSatisfy: [:each | enumerated includes: each]).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:24'!
testEquals
	| set |

	set := self emptySet.
	self
		assertSet: set equals: self emptySet;
		denySet: set equals: self setWithFunctions;
		denySet: set equals: self setWithDisjointFunctions.

	set := self setWithFunctions.
	self
		assertSet: set equals: self setWithFunctions;
		denySet: set equals: self setWithDisjointFunctions.

	set := self setWithDisjointFunctions.
	self assertSet: set equals: self setWithDisjointFunctions.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 11/18/2015 05:25'!
testFunctionNamed
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert: (set functionNamed: each functionName) == each].

	self
		shouldnt: [set functionNamed: self absentFunction functionName]
		raise: Error.
	self assert: (set functionNamed: self absentFunction functionName) isNil.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 2/10/2015 19:14'!
testFunctionNamedIfAbsent
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert:
			(set
				functionNamed: each functionName
				ifAbsent: [self absentFunction]) == each].
	self assert:
		(set
			functionNamed: self absentFunction functionName
			ifAbsent: [self absentFunction]) == self absentFunction.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 16:34'!
testIncludes
	| set |

	set := self emptySet.
	self deny: (self functions anySatisfy: [:each | set includes: each]).

	set := self setWithFunctions.
	self assert: (self functions allSatisfy: [:each | set includes: each]).
	self deny: (self disjointFunctions anySatisfy: [:each | set includes: each]).

	set := self setWithDisjointFunctions.
	self assert: (self disjointFunctions allSatisfy: [:each | set includes: each]).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 2/10/2015 19:17'!
testIncludesFunctionNamed
	| set |

	set := self emptySet.
	self functions do: [:each |
		self deny: (set includesFunctionNamed: each functionName)].

	set := self setWithFunctions.
	self functions do: [:each |
		self assert: (set includesFunctionNamed: each functionName)].
	self disjointFunctions do: [:each |
		self deny: (set includesFunctionNamed: each functionName)].

	set := self setWithDisjointFunctions.
	self disjointFunctions do: [:each |
		self assert: (set includesFunctionNamed: each functionName)].! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:58'!
testMinusFunction
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self
			should: [set - each]
			subtract: each
			from: set
			equaling: (self setClass newFrom: (self functions copyWithout: each))].

	self disjointFunctions do: [:each |
		self
			should: [set - each]
			subtract: each
			from: set
			equaling: self setWithFunctions].

	self assert:
		(self functions first - self functions second) =
			(self setClass with: self functions first).
	self assert:
		(self functions first - self functions first) =
			self emptySet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:50'!
testMinusSet
	| firstSet secondSet |

	firstSet := self setWithFunctions.
	secondSet := self emptySet.
	self
		should: [firstSet - secondSet]
		subtract: secondSet
		from: firstSet
		equaling: self setWithFunctions.

	secondSet := self setWithDisjointFunctions.
	self
		should: [firstSet - secondSet]
		subtract: secondSet
		from: firstSet
		equaling: self setWithFunctions.
	self
		should: [firstSet - firstSet]
		subtract: firstSet
		from: firstSet
		equaling: self emptySet.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:59'!
testPlusFunction
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self
			should: [set + each]
			add: each
			and: set
			equaling: self setWithFunctions].

	set := self setWithFunctions.
	self disjointFunctions do: [:each |
		self
			should: [set + each]
			add: each
			and: set
			equaling: (self setClass newFrom: (self functions copyWith: each))].

	self assert:
		(self functions first + self functions second) =
			(self setClass
				with: self functions first
				with: self functions second).

	self assert:
		(self functions first + self functions first) =
			(self setClass with: self functions first).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:44'!
testPlusSet
	| firstSet secondSet |

	firstSet := self setWithFunctions.
	secondSet := self emptySet.
	self
		should: [firstSet + secondSet]
		add: firstSet
		and: secondSet
		equaling: self setWithFunctions.
	self
		should: [firstSet + firstSet]
		add: firstSet
		and: firstSet
		equaling: self setWithFunctions.

	secondSet := self setWithDisjointFunctions.
	self
		should: [firstSet + secondSet]
		add: firstSet
		and: secondSet
		equaling:
			(self setClass newFrom:
				self setWithDisjointFunctions, self setWithFunctions).! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/20/2015 14:00'!
testPostCopy
	| set copy |

	set := self setWithFunctions.
	copy := set copy.
	self
		assert: set = copy;
		assert: set functionsByName = copy functionsByName;
		deny: set functionsByName == copy functionsByName.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/19/2015 15:27'!
testRemoveIfAbsent
	| set |

	set := self setWithFunctions.
	self functions do: [:each |
		self assert:
			(set
				remove: each
				ifAbsent: [self absentFunction]) == each.
		self
			assert:
				(set
					remove: each
					ifAbsent: [self absentFunction]) == self absentFunction.
		self deny: (set includes: each).
		self
			should: [set remove: each]
			raise: Error].
	self assert: set isEmpty.! !

!XPathFunctionSetTest methodsFor: 'tests' stamp: 'monty 1/29/2016 15:32'!
testSize
	| set size |

	set := self emptySet.
	size := 0.
	self assert: set size = size.
	self functions do: [:each |
		set add: each.
		size := size + 1.
		self assert: set size = size].! !

TestCase subclass: #XPathTest
	instanceVariableNames: 'document namespacedDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPath-Tests'!

!XPathTest methodsFor: 'asserting' stamp: 'monty 10/7/2015 07:29'!
assertNodeSet: aNodeSet hasNodes: aNodeCollection
	self
		assert: aNodeSet size = aNodeCollection size;
		assert: (aNodeCollection allSatisfy: [:each | aNodeSet includes: each])! !

!XPathTest methodsFor: 'asserting' stamp: 'monty 10/11/2015 02:45'!
assertNodeSet: aNodeSet hasSortOrder: aSortOrder
	self assert: (aNodeSet sortOrder class == aSortOrder)! !

!XPathTest methodsFor: 'running' stamp: 'monty 3/3/2015 12:48'!
setUp
	document :=
		(XMLDOMParser on: XMLParserTest addressBookXML)
			preservesCommentNodes: true;
			parseDocument.
	namespacedDocument := 
		(XMLDOMParser on: XMLDOMParserTest addressBookNamespacePrefixedXML)
			preservesCommentNodes: true;
			parseDocument.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/16/2015 01:22'!
testAbbreviatedSteps
	| result |

	result := XPathNodeSet with: document.
	self
		assert: (document xPath: '.') = result;
		assert: (document xPath: './.') = result;
		assert: (document xPath: './..') isEmpty;
		assert: (document xPath: '..') isEmpty;
		assert: (document xPath: '../..') isEmpty;
		assert: (document xPath: '/.') = result;
		assert: (document xPath: '/./.') = result;
		assert: (document xPath: '/./..') isEmpty;
		assert: (document xPath: '/..') isEmpty;
		assert: (document xPath: '/../..') isEmpty.

	result := document allElementsNamed: 'person'.
	self
		assert: (document xPath: './addressbook/./person/.') = result;
		assert: (document xPath: '././addressbook/././person/./.') = result;
		assert: (document xPath: './addressbook/../addressbook/person/../person') = result;
		assert: (document xPath: './addressbook/./../addressbook/./person/./../person') = result.

	"abbreviated steps can't have predicates"
	self
		should: [document xPath: '.[true()]']
		raise: XPathSyntaxException.
	self
		should: [document xPath: '..[true()]']
		raise: XPathSyntaxException.
	self
		should: [document xPath: '/.[true()]']
		raise: XPathSyntaxException.
	self
		should: [document xPath: '/..[true()]']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/15/2015 06:16'!
testAbsoluteLocationPaths
	document allNodesDo: [:each |
		self
			assert: (XPath for: '/' in: each) = (XMLNodeList with: document);
			assert: (XPath for: '/*' in: each) = (XMLNodeList with: document root);
			assert: (XPath for: '/self::node()' in: each) = (XMLNodeList with: document);
			assert: (XPath for: '/addressbook/person' in: each) = (document allElementsNamed: 'person')]! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/19/2015 00:30'!
testAddFunction
	| path functionsWithoutCeiling context |

	path := XPath for: 'ceiling(1.9)'.

	functionsWithoutCeiling :=
		XPathFunctionSet default copyWithout: XPathCeilingFunction.
	context := XPathContext functions: functionsWithoutCeiling.
	self deny: (context includesFunction: XPathCeilingFunction).
	self
		should: [
			path
				in: nil
				context: context]
		raise: XPathUndeclaredFunctionException.

	(context := XPathContext functions: functionsWithoutCeiling)
		addFunction: XPathCeilingFunction.
	self assert: (context includesFunction: XPathCeilingFunction).
	"adding should modify an internal copy, not the original"
	self deny: (functionsWithoutCeiling includes: XPathCeilingFunction).
	self assert:
		(path
			in: nil
			context: context) = 2.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/20/2015 23:03'!
testAncestorAxis
	| ancestors |

	ancestors := document allNodesSelect: [:each | each hasChildren].
	self
		assertNodeSet: (XPath for: '//ancestor::node()' in: document)
		hasNodes: ancestors.

	ancestors := ancestors select: [:ancestor |
		ancestor nodes anySatisfy: [:each | each hasChildren]].
	self
		assertNodeSet: (XPath for: '//ancestor::node()/ancestor::node()' in: document)
		hasNodes: ancestors.

	ancestors
		removeAll;
		addAll: (document allElementsNamed: 'person');
		addAll: ancestors last ancestorNodes.
	self
		assertNodeSet: (XPath for: '*/person/node()/ancestor::node()' in: document)
		hasNodes: ancestors.

	ancestors
		removeAll;
		addLast: document.
	self
		assertNodeSet: (XPath for: 'addressbook/ancestor::node()' in: document)
		hasNodes: ancestors.

	ancestors
		removeAll;
		addLast: document root.
	self
		assertNodeSet: (XPath for: 'addressbook/person/ancestor::*' in: document)
		hasNodes: ancestors.

	self
		assert: (XPath for: 'addressbook/ancestor::*' in: document) isEmpty;
		assert: (XPath for: 'ancestor::node()' in: document) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/20/2015 17:41'!
testAncestorOrSelfAxis
	| nodes |

	nodes := document allNodesSelect: [:each | each hasChildren not].
	self
		assertNodeSet:
			(XPath
				for: '//node() [ count( child::node() ) = 0]'
				in: document)
		hasNodes: nodes.

	nodes := document allNodes.
	self
		assertNodeSet:
			(XPath
				for: '//node( )[count(child::node( ))=0]/ancestor-or-self::node( )'
				in: document)
		hasNodes: nodes.

	nodes := (document allElementsNamed: 'person') allButLast.
	nodes addAll: nodes last ancestorNodes.
	self
		assertNodeSet:
			(XPath
				for: '//person[last()]/preceding-sibling::person/ancestor-or-self::node()'
				in: document)
		hasNodes: nodes.

	nodes removeAll.
	(document allElementsNamed: 'person') do: [:each |
		nodes
			addLast: each;
			addAll: each nodes].
	nodes addAll: nodes first ancestorNodes.
	self
		assertNodeSet: (XPath for: '*/person/node()/ancestor-or-self::node()' in: document)
		hasNodes: nodes.

	nodes
		removeAll;
		addLast: document root;
		addAll: (document allElementsNamed: 'person').
	self
		assertNodeSet: (XPath for: 'addressbook/person/ancestor-or-self::*' in: document)
		hasNodes: nodes.

	nodes
		removeAll;
		addLast: document root.
	self
		assertNodeSet: (XPath for: 'addressbook/ancestor-or-self::*' in: document)
		hasNodes: nodes.

	nodes
		removeAll;
		addLast: document.
	self
		assertNodeSet: (XPath for: 'ancestor-or-self::node()' in: document)
		hasNodes: nodes.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/7/2015 20:25'!
testAsXPath
	| path |

	path := '/addressbook' asXPath.
	self
		assert: (path isKindOf: XPath);
		assert: (document xPath: path) = (XPathNodeSet with: document root);
		assert: path asXPath == path.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/10/2015 23:57'!
testAttributeAxis
	| result expected |

	result :=
		XPath
			for: 'addressbook/person/address/@city'
			in: document.
	self assert: result size = 4.
	result do: [:each |
		self
			assert: each class == XMLAttribute;
			assert: each name = 'city'].
	self assert:
		(result collect: [:each | each value]) asArray =
			#('Los Angeles' 'New York' 'Los Angeles' 'New York').

	"attribute nodes don't have attributes"
	self assert: 
		(XPath
			for: 'addressbook/person/address/@city/@city'
			in: document) isEmpty.

	expected := XMLNodeList new.
	(document allElementsNamed: 'person') allButLast do: [:each |
		expected addAll: each attributeNodes].
	result :=
		(document allElementsNamed: 'person') last xPath:
			'preceding-sibling::node()/attribute::node()'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/15/2015 06:44'!
testAttributeNameTest
	| results |

	results :=
		document xPath:
			'addressbook/person/attribute::node()/self::attribute(employee-number)'.
	self
		assert: results size = 4;
		assert:
			results =
				((document allElementsNamed: 'person') collect: [:each |
					each attributeNodeAt: 'employee-number']).

	results :=
		document xPath:
			'addressbook/person/attribute::node()/self::attribute(absent)'.
	self assert: results isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/20/2015 05:29'!
testAttributePredicates
	| persons |

	persons := (document allElementsNamed: 'person') copyFrom: 2 to: 3.
	self
		assertNodeSet: (document xPath: 'addressbook/person[ @ middle-initial ]')
		hasNodes: persons;
		assertNodeSet: (document xPath: 'addressbook/person[ @ middle-initial and position() > 1 ]')
		hasNodes: persons;
		assertNodeSet: (document xPath: 'addressbook/person[ @ * [ local-name() = "middle-initial" ] ]')
		hasNodes: persons;
		assertNodeSet: (document xPath: 'addressbook/person[ @ * [ self::attribute(middle-initial) ] ]')
		hasNodes: persons;
		assertNodeSet:
			(document xPath:
				'addressbook/person[ @ * [ local-name() = "middle-initial" ] and position() < 4 ]')
		hasNodes: persons.

	self
		assert: (document xPath: 'addressbook/person[ @ middle-initial and position() < 2 ]') isEmpty;
		assert:
			(document xPath:
				'addressbook/person[ @ * [ local-name() = "middle-initial" ] and position() > 3 ]') isEmpty;
		assert: (document xPath: 'addressbook/person[ @ absent ]') isEmpty;
		assert: (document xPath: 'addressbook/person[ @ * [ self::attribute(absent) ] ]') isEmpty;
		assert: (document xPath: 'addressbook/person[ @ * [ local-name() = "absent" ] ]') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/19/2015 22:02'!
testAttributePredicatesComparison
	| addresses |

	addresses := document allElementsNamed: 'address'.
	self
		assertNodeSet: (document xPath: '//address[ @ number >= 344 ]')
		hasNodes: addresses.

	addresses := addresses copyReplaceFrom: 2 to: 2 with: #().
	self
		assertNodeSet: (document xPath: '//address[ @ number > 344 ]')
		hasNodes: addresses.

	addresses := (document allElementsNamed: 'address') copyFrom: 2 to: 2.
	self
		assertNodeSet: (document xPath: '//address[ @ number <= 344 ]')
		hasNodes: addresses.
	self assert: (document xPath: '//address[ @ number < 344 ]') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/19/2015 22:54'!
testAttributePredicatesEquals
	| persons addresses |

	persons := (document allElementsNamed: 'person') copyFrom: 1 to: 1.
	self
		assertNodeSet: (document xPath: 'addressbook/person[ @ employee-number = "A0000" ]')
		hasNodes: persons;
		assertNodeSet: (document xPath: 'addressbook/person[ @ employee-number = ''A0000'' ]')
		hasNodes: persons;
		assertNodeSet:
			(document xPath: 'addressbook/person[ @ employee-number = "A0000" and position() = 1]')
		hasNodes: persons.
	self assert:
		(document xPath: 'addressbook/person[ @ employee-number = ''A0000'' and position() > 1]') isEmpty.

	addresses := (document allElementsNamed: 'address') copyFrom: 2 to: 2.
	self
		assertNodeSet: (document xPath: '//address[ @ number = 344 ]')
		hasNodes: addresses.
	self assert:
		(document xPath: '//address[ @ number = 1 ]') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/19/2015 22:54'!
testAttributePredicatesNotEquals
	| persons addresses |

	persons := (document allElementsNamed: 'person') allButFirst.
	self
		assertNodeSet: (document xPath: 'addressbook/person[ @ employee-number !!= "A0000" ]')
		hasNodes: persons;
		assertNodeSet: (document xPath: 'addressbook/person[ @ employee-number !!= ''A0000'' ]')
		hasNodes: persons;
		assertNodeSet:
			(document xPath: 'addressbook/person[ @ employee-number !!= "A0000"  and position() > 1]')
		hasNodes: persons.
	self assert:
		(document xPath:
			'addressbook/person[ @ employee-number !!= ''A0000'' and position() = 1]') isEmpty.

	addresses := document allElementsNamed: 'address'.
	self
		assertNodeSet: (document xPath: '//address[ @ number !!= 1 ]')
		hasNodes: addresses.
	addresses := addresses copyReplaceFrom: 2 to: 2 with: #().
	self
		assertNodeSet: (document xPath: '//address[ @ number !!= 344 ]')
		hasNodes: addresses.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 15:01'!
testAttributeTypeTest
	| results |

	results := XMLNodeList new.
	document allNodesDo: [:each |
		each isElement
			ifTrue: [results addAll: each attributeNodes]
			ifFalse: [
				each isStringNode
					ifTrue: [results addLast: each]]].
	self assert: (document xPath: '//attribute::* | //text()') = results.
	self assert:
		(document xPath: '(//attribute::* | //text())/ self :: attribute()') =
			(results select: [:each | each isAttribute]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 01:48'!
testChildAxis
	| nodes |

	nodes := document allElementsNamed: 'person'.
	self
		assertNodeSet:
			(XPath
				for: 'child::addressbook/child::person'
				in: document)
		hasNodes: nodes.

	nodes removeAll.
	document allElementsNamed: 'person' do: [:each | nodes addAll: each nodes].
	self
		assertNodeSet:
			(XPath
				for: '/addressbook/person/node()'
				in: document)
		hasNodes: nodes.

	nodes := document allElementsNamed: 'person'.
	nodes third nodesDo: [:each | nodes addAll: each nodes].
	self
		assertNodeSet:
			(XPath
				for: '(/addressbook | //person[3]/*)/node()'
				in: document)
		hasNodes: nodes.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/23/2015 11:07'!
testCommentTypeTest
	| comments |

	comments := document xPath: '//comment()'.
	self
		assert: comments size = 2;
		assert: comments = (document allNodesSelect: [:each | each isComment]);
		assert: (document xPath: '//comment()/self::comment()') = comments;
		assert: (document xPath: '//comment()/comment()') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/19/2015 06:25'!
testConcatFunction
	self
		assert: (XPath for: 'concat("one", "two")' in: nil) = 'onetwo';
		assert: (XPath for: 'concat("one", "two", true())' in: nil) = 'onetwotrue'.
	self
		should: [XPath for: 'concat("one")' in: nil]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:04'!
testCountFunction
	self assert:
		(XPath for: 'count(addressbook/person)' in: document) =
			(document allElementsNamed: 'person') size.
	self assert:
		(XPath for: 'addressbook/person[count(@*) >= 4]' in: document) =
			((document allElementsNamed: 'person') copyFrom: 2 to: 3).
	self
		should: [XPath for: 'count()' in: nil]
		raise: XPathArgumentNumberException.
	self
		should: [XPath for: 'count(1)' in: nil]
		raise: XPathTypeException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 09:47'!
testDescendantAxis
	| path results |

	path := XPath for: '//address/@city'.
	results := path in: document.
	self
		assert: results size = 4;
		assert: results first value = 'Los Angeles'.

	path := XPath for: 'addressbook/person//address'.
	self
		assertNodeSet: (path in: document)
		hasNodes: (document allElementsNamed: 'address').

	path := XPath for: 'addressbook/person//address//absent'.
	self assert: (path in: document) isEmpty.

	path := XPath for: 'addressbook/person//*'.
	self
		assertNodeSet: (path in: document)
		hasNodes:
			(document root descendentElementsSelect: [:each | (each isNamed: 'person') not]).

	path := XPath for: 'addressbook/person//*/descendant::*'.
	self
		assertNodeSet: (path in: document)
		hasNodes:
			(document root descendentElementsSelect: [:each |
				((each isNamed: 'person')
					or: [each parent isNamed: 'person']) not]).

	path := XPath for: 'addressbook/person[last()]/preceding-sibling::person/descendant::*'.
	self
		assertNodeSet: (path in: document)
		hasNodes:
			(document root descendentElementsSelect: [:each |
				((each isNamed: 'person')
					or: [each ancestorElements includes: document root lastElement]) not]).

	path := XPath for: 'addressbook/descendant::comment()'.
	self
		assertNodeSet: (path in: document)
		hasNodes: (document allNodesSelect: [:each | each isComment]).

	path := XPath for: 'addressbook/descendant::comment()/descendant::node()'.
	self assert: (path in: document) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:21'!
testDescendantOrSelfAxis
	self
		assertNodeSet: (XPath for: '//.' in: document)
		hasNodes: document allNodes.
	self
		assertNodeSet: (XPath for: '//./node()' in: document)
		hasNodes: document descendentNodes.
	self
		assertNodeSet: (XPath for: '//~node()' in: document)
		hasNodes: document allNodes.
	self
		assertNodeSet: (XPath for: '// self :: node()' in: document)
		hasNodes: document allNodes.
	self
		assertNodeSet: (XPath for: '/ descendant-or-self ::node()' in: document)
		hasNodes: document allNodes.
	self
		assertNodeSet: (XPath for: '//*' in: document)
		hasNodes: document allElements.
	self
		assertNodeSet:
			(XPath for: '//comment()/descendant-or-self::comment()' in: document)
		hasNodes: (document allNodesSelect: [:each | each isComment]).
	self
		assertNodeSet:
			(XPath for: 'addressbook/person/descendant-or-self::element()' in: document)
		hasNodes: document root descendentElements.
	self
		assertNodeSet:
			(XPath for: 'addressbook/person/descendant-or-self::address' in: document)
		hasNodes: (document allElementsNamed: 'address').
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person[last()]/preceding-sibling::person/descendant-or-self::*'
				in: document)
		hasNodes: (document root descendentElements copyUpTo: document root lastElement).
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/contact-info/comment()/descendant-or-self::node()'
				in: document)
		hasNodes: (document allNodesSelect: [:each | each isComment]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/16/2015 03:38'!
testDocumentOrderLosing
	| expected result |

	expected := document allNodesSelect: [:each | each hasChildren].
	result := document xPath: 'descendant::node()/ancestor::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected := document allNodes.
	result := document xPath: 'descendant::node()/ancestor-or-self::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected := document root descendentNodesSelect: [:each | each previousNode isNil].
	result := document xPath: 'descendant::node()/descendant::node()[1]'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected removeAll.
	document descendentNodesDo: [:each |
		each hasChildren
			ifTrue: [expected add: each firstNode]].
	result := document xPath: 'descendant::node()/descendant-or-self::node()[2]'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected := document root descendentNodesSelect: [:each | each hasParent].
	result := document xPath: 'descendant::node()/node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected := document root descendentNodesSelect: [:each | each previousNode notNil].
	result := document xPath: 'descendant::node()/following-sibling::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected :=
		document root firstNode descendentNodesSelect: [:each | each previousNode notNil].
	document root nodes allButFirst do: [:each | expected addAll: each allNodes].
	result := document xPath: '/descendant::node()/following::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected := document root descendentNodesSelect: [:each | each nextNode notNil].
	result := document xPath: 'descendant::node()/preceding-sibling::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.

	expected :=
		document root nodes last descendentNodesSelect: [:each |
			each nextNode notNil
				or: [each parent nextNode notNil]].
	document root nodes allButLast do: [:each | expected addAll: each allNodes].
	result := document xPath: 'descendant::node()/preceding::node()'.
	self
		assertNodeSet: result hasNodes: expected;
		assertNodeSet: result hasSortOrder: XPathUnknownSortOrder.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/15/2015 11:13'!
testDocumentOrderPreserving
	| expected result |

	expected := document allNodesSelect: [:each | each hasChildren].
	result := document xPath: 'descendant::node()/parent::node()[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected := document allNodes allButFirst.
	result := document xPath: 'descendant::node()/self::node()[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	document allElementsDo: [:each | expected addAll: each attributeNodes].
	result := document xPath: 'descendant::*/@*[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	namespacedDocument allElementsDo: [:each | expected addAll: each namespaceNodes].
	result := namespacedDocument xPath: 'descendant::*/@@*[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:21'!
testDot
	| path results |

	path := XPath for: 'addressbook/person/address/./@city'.
	results := path in: document.
	self assert: results size = 4.
	self assert: (results at: 1) value = 'Los Angeles'.
! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:21'!
testDotDot
	| path results |
	path := XPath for: 'addressbook/person/address/../address/@city'.
	results := path in: document.
	self assert: results size = 4.
	self assert: (results at: 1) value = 'Los Angeles'.
! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:21'!
testDotDotTooHigh
	| path results |
	path := XPath for: 'person/address/../../../address/@city'.
	results := path in: document.
	self assert: results isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 11:19'!
testDoubleWildcard
	self
		assertNodeSet: (XPath for: 'addressbook/*:*' in: document)
		hasNodes: (document allElementsNamed: 'person').
	self
		assertNodeSet: (XPath for: 'book:addressbook/*:*' in: namespacedDocument)
		hasNodes: (namespacedDocument allElementsNamed: 'person').
	self
		assertNodeSet: 
			(XPath
				for: '*:*/*:*'
				in: namespacedDocument
				context: (XPathContext new declarePrefix: 'two' uri: 'personNS'))
		hasNodes: (namespacedDocument allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 11:21'!
testElementDoubleWildcard
	self
		assertNodeSet: (XPath for: 'addressbook/element(*:*)' in: document)
		hasNodes: (document allElementsNamed: 'person').
	self
		assertNodeSet: (XPath for: 'book:addressbook/element(*:*)' in: namespacedDocument)
		hasNodes: (namespacedDocument allElementsNamed: 'person').
	self
		assertNodeSet:
			(XPath
				for: 'element(*:*)/element(*:*)'
				in: namespacedDocument
				context: (XPathContext new declarePrefix: 'two' uri: 'personNS')) 
		hasNodes: (namespacedDocument allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:25'!
testElementExpandedNameTest
	self
		assertNodeSet:
			(namespacedDocument xPath: 'element( Q{bookNS}addressbook ) / element( Q{personNS}person )')
		hasNodes: (namespacedDocument allElementsNamed: 'person').
	self assert: (namespacedDocument xPath: 'element(Q{undeclared}addressbook)') isEmpty.

	self
		should: [namespacedDocument xPath: 'element(Q{bookNS} addressbook)']
		raise: XPathSyntaxException.
	self
		should: [namespacedDocument xPath: 'element(Q {bookNS}addressbook)']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:26'!
testElementExpandedNameWildcardTest
	self
		assertNodeSet: (namespacedDocument xPath: 'element( Q{bookNS}* ) / element( Q{personNS}* )')
		hasNodes: (namespacedDocument allElementsNamed: 'person').
	self assert: (namespacedDocument xPath: 'element(Q{undeclared}*)') isEmpty.

	self
		should: [namespacedDocument xPath: 'element(Q{bookNS} *)']
		raise: XPathSyntaxException.
	self
		should: [namespacedDocument xPath: 'element(Q {bookNS}*)']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:27'!
testElementLocalNameWildcardTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assertNodeSet: (namespacedDocument xPath: 'element(book:*)/element(ps:*)')
		hasNodes: persons.
	self assert: (namespacedDocument xPath: 'element(book:*)/element(undeclared:*)') isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'element(one:*)/element(two:*)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))
		hasNodes: persons.
	self assert:
		(XPath
			for: 'element(one:*)/element(one:*)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'element(undeclared:*)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:28'!
testElementNameTest
	self
		assertNodeSet: (document xPath: 'element(addressbook)/element(person)')
		hasNodes: (document allElementsNamed: 'person').
	self assert:
		(namespacedDocument xPath: 'element(addressbook)/element(person)') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:29'!
testElementPrefixWildcardTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self
		assertNodeSet: (namespacedDocument xPath: 'element(*:addressbook)/element(*:person)')
		hasNodes: persons.
	self assert: (namespacedDocument xPath: 'element(*:addressbook)/element(*:contaxt-info)') isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'element(one:addressbook)/element(*:person)'
				in: namespacedDocument
				context: (XPathContext new declarePrefix: 'one' uri: 'bookNS'))
		hasNodes: persons.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(*:contact-info)'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:30'!
testElementQualfiedNameTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assertNodeSet:
			(namespacedDocument xPath:
				'element(book:addressbook)/element(ps:person)')
		hasNodes: persons.
	self assert:
		(namespacedDocument xPath:
			'element(book:addressbook)/element(undeclared:person)') isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'element(one:addressbook)/element(two:person)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))
		hasNodes: persons.
	self assert:
		(XPath
			for: 'element(one:addressbook)/element(one:person)'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.

	self
		should: [
			(XPath
				for: 'element(undeclared:addressbook)'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:30'!
testElementTypeTest
	self
		assertNodeSet: (document xPath: '//element()')
		hasNodes: document allElements.
	self
		assertNodeSet: (document xPath: '//node()/self::element()')
		hasNodes: document allElements! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:31'!
testExpandedNameTest
	self
		assertNodeSet:
			(namespacedDocument xPath: 'Q{bookNS}addressbook/Q{personNS}person')
		hasNodes: (namespacedDocument allElementsNamed: 'person').
	self assert: (namespacedDocument xPath: 'Q{undeclared}addressbook') isEmpty.

	self
		should: [namespacedDocument xPath: 'Q{bookNS} addressbook']
		raise: XPathSyntaxException.
	self
		should: [namespacedDocument xPath: 'Q {bookNS}addressbook']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 03:32'!
testFalseFunction
	self deny: 'false()' asXPath evaluate.
	self
		should: ['false(1)' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/28/2015 16:06'!
testFileterExpressionPredicates
	self
		assertNodeSet:
			(document xPath: '(./addressbook/*)[position() <= 3 and position() > 1]')
		hasNodes: ((document allElementsNamed: 'person') copyFrom: 2 to: 3).
	self
		assertNodeSet: (document xPath: '(addressbook/*)[true()] / address')
		hasNodes: (document allElementsNamed: 'address').
	self
		assertNodeSet:
			(document xPath: '(addressbook)[true()] // email')
		hasNodes: (document allElementsNamed: 'email').
	self
		assertNodeSet: (document xPath: '(//person[position()>1])[position()<=2]')
		hasNodes: ((document allElementsNamed: 'person') copyFrom: 2 to: 3).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 04:34'!
testFollowingAxis
	| persons following |

	persons := document allElementsNamed: 'person'.
	following := XMLNodeList new.
	persons allButFirst do: [:person |
		person allNodesDo: [:each | following addLast: each]].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/following::node()'
				in: document)
		hasNodes: following.

	following removeAll.
	persons do: [:person |
		 person nodes allButFirst do: [:node |
			node allNodesDo: [:each | following addLast: each]]].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/node()/following::node()'
				in: document)
		hasNodes: following.

	following removeAll.
	persons first descendentNodesDo: [:each |
		each previousNode
			ifNotNil: [following addLast: each]].
	persons allButFirst do: [:person | following addAll: person allNodes].
	self
		assertNodeSet: (XPath for: '//following::node()' in: document)
		hasNodes: following.

	following removeAll.
	persons allButFirst do: [:person |
		person allNodesDo: [:each | following addLast:each]].
	self
		assertNodeSet:
			(XPath
				for: '(//person[1]|//person[3]/node()[1])/following::node()'
				in: document)
		hasNodes: following.
	self
		assertNodeSet:
			(XPath
				for: '(//person[1]|//person[last()])/following::node()'
				in: document)
		hasNodes: following.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 05:07'!
testFollowingSiblingAxis
	| persons siblings |

	persons := document allElementsNamed: 'person'.
	siblings := persons allButFirst.
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/following-sibling::person'
				in: document)
		hasNodes: siblings.

	siblings := XMLNodeList new.
	persons do: [:each | siblings addAll: each nodes allButFirst].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/node()/following-sibling::node()'
				in: document)
		hasNodes: siblings.

	siblings := persons first descendentNodesSelect: [:each | each previousNode notNil].
	siblings addAll: persons allButFirst.
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person[1]//following-sibling::node()'
				in: document)
		hasNodes: siblings.

	siblings := document allNodesSelect: [:each |
		each hasParent
			and: [each previousNode notNil]].
	self
		assertNodeSet:
			(XPath
				for: '//following-sibling::node()'
				in: document)
		hasNodes: siblings.

	siblings := persons allButFirst.
	siblings addAll: persons third nodes allButFirst.
	self
		assertNodeSet:
			(XPath
				for: '(//person[1]|//person[3]/node()[1])/following-sibling::node()'
				in: document)
		hasNodes: siblings.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 15:34'!
testFunctions
	| functionSet |

	functionSet := XPathFunctionSet default.
	self
		assert: XPathContext new functions = functionSet;
		deny: XPathContext new functions == functionSet;
		assert: (XPathContext functions: functionSet) functions == functionSet;
		assert: (XPathContext new functions: functionSet) functions == functionSet.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/8/2015 12:10'!
testIDFunction
	| xmlWithIds documentWithIds idElements |

	xmlWithIds :=
		'<!!DOCTYPE one [
			<!!ELEMENT one ANY>
			<!!ATTLIST one id ID #REQUIRED>
			<!!ELEMENT two ANY>
			<!!ATTLIST two id2 ID #IMPLIED>
			<!!ELEMENT three ANY>
			<!!ATTLIST three
				id ID #REQUIRED
				id3 CDATA #IMPLIED>
			<!!ELEMENT four ANY>
			<!!ATTLIST four id4 ID #REQUIRED>
			<!!ELEMENT five ANY>
			<!!ATTLIST five id CDATA #IMPLIED>
			<!!ELEMENT six ANY>
			<!!ATTLIST six id2 CDATA #IMPLIED>
			<!!ELEMENT seven ANY>
			<!!ATTLIST seven id4 CDATA #IMPLIED>
		]>
		<one id="first"><two/><three id="third" id3="non-id"><four id4="fourth">
			<five id="non-id">first</five>
			<six id2="non-id">third</six>
			<seven id4="non=id">non-id</seven>
		</four></three></one>'.
	documentWithIds :=
		XMLDOMParser
			parse: xmlWithIds
			usingNamespaces: true
			validation: true.

	idElements :=
		documentWithIds allElements select: [:each |
				each isNamedAny: #('one' 'three')].
	self
		assert: (documentWithIds xPath: 'id("  first  third  non-id  ")') = idElements;
		assert: (documentWithIds xPath: 'id("first third non-id")') = idElements;
		assert: (documentWithIds xPath: 'id(//*)') = idElements.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/2/2015 10:06'!
testInvalidAbbreviatedLocationPath
	self
		should: ['/ /one' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['one/ /two' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['//' asXPath compile]
		raise: XPathSyntaxException.
	self
		should: ['//one/' asXPath compile]
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/30/2015 15:56'!
testInvalidFilterExpressionType
	#('1[true()]' '"string"[true()]' 'true()[true()]'
		'(1)[true()]' '("string")[true()]' '(true())[true()]'
		'1/addressbook' '"string"/addressbook' 'true()/addressbook'
		'(1)/addressbook' '("string")/addressbook' '(true())/addressbook'
	) do: [:each |
		self
			should: [document xPath: each]
			raise: XPathTypeException]! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/12/2015 12:55'!
testInvalidNumberTokenizing
	self
		should: ['. 1' asXPath evaluate]
		raise: XPathSyntaxException.
	self
		should: ['1 .' asXPath evaluate]
		raise: XPathSyntaxException.
	self
		should: ['.1.2' asXPath evaluate]
		raise: XPathSyntaxException.
	self
		should: ['1.2.' asXPath evaluate]
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/30/2015 14:33'!
testInvalidUnionExpressionType
	#('//person | 1' '1 | //person'
		'//person | true()' 'true() | //person'
		'//person | "string"' '"string" | //person') do: [:each |
		self
			should: [document xPath: each]
			raise: XPathTypeException].! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:02'!
testLangFunction
	| element |

	(element := XMLElement named: 'test')
		attributeAt: 'xml:lang' put: 'en-us'.
	self
		assert: ('lang("En")' asXPath in: element);
		assert: ('lang("en-US")' asXPath in: element);
		deny: ('lang("en-UK")' asXPath in: element);
		deny: ('lang("DE")' asXPath in: element).

	element attributeAt: 'xml:lang' put: 'En'.
	self
		assert: ('lang("En")' asXPath in: element);
		deny: ('lang("en-US")' asXPath in: element);
		deny: ('lang("en-UK")' asXPath in: element);
		deny: ('lang("DE")' asXPath in: element).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:39'!
testLastFunction
	| path elements |

	path := XPath for: 'last()'.
	self assert: (path in: document) = 1.

	path := XPath for: 'last()'.
	elements := document allElementsNamed: 'person'.
	elements do: [:each | self assert: (path in: each) = 1].

	path := XPath for: 'addressbook/person[last()]'.
	self
		assertNodeSet: (path in: document)
		hasNodes: (XMLNodeList with: elements last).

	path := XPath for: 'addressbook/descendant::node()[last()]'.
	self
		assertNodeSet: (path in: document)
		hasNodes: (XMLNodeList with: document allNodes last).

	path := XPath for: 'addressbook//node()[last()]'.
	self
		assertNodeSet: (path in: document)
		hasNodes: (document root descendentNodesSelect: [:each | each nextNode isNil]).

	self
		should: ['last(0)' asXPath evaluate]
		raise: XPathException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/18/2015 16:17'!
testLocalNameFunction
	self assert:
		(XPath
			for: 'local-name(addressbook/person[1]/*[last()]/preceding::*)'
			in: document) = 'contact-info'.
	self assert:
		(XPath
			for: 'local-name(addressbook/absent)'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'local-name(//comment())'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'local-name()'
			in: document root) = 'addressbook'.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 04:08'!
testLocalNameWildcardTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assertNodeSet: (namespacedDocument xPath: 'book:*/ps:*')
		hasNodes: persons.
	self assert: (namespacedDocument xPath: 'book:*/undeclared:*') isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'one:*/two:*'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))
		hasNodes: persons.
	self assert:
		(XPath
			for: 'one:*/one:*'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.

	self
		should: [
			(XPath
				for: 'undeclared:*'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/28/2015 16:05'!
testLocationPathPredicates
	self
		assertNodeSet:
			(document xPath: '/descendant::home-phone[1][2 - 1][position() = 1]')
		hasNodes:
			(XMLNodeList with: (document allElementsNamed: 'home-phone') first).
	self assert:
		(document xPath: '/descendant::home-phone[1][position() + 1]') isEmpty.

	self
		assertNodeSet:
			(document xPath: '//home-phone[1][1 + 0][position() = 1]')
		hasNodes:
			(document allElementsNamed: 'home-phone') allButLast.
	self
		assertNodeSet:
			(document xPath: '//home-phone[1 + 1][1][position() = 1]')
		hasNodes:
			(XMLNodeList with: (document allElementsNamed: 'home-phone') third).
	self assert:
		(document xPath: '//home-phone[position() = 1][1 + 1]') isEmpty.

	self
		assertNodeSet:
			(document xPath: '/descendant::home-phone[2][1 + 0][position() = 1]')
		hasNodes:
			(XMLNodeList with: (document allElementsNamed: 'home-phone') second).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/11/2015 00:20'!
testLogicalOperators
	self
		deny: '0 or 0 or 0' asXPath evaluate;
		assert: '0 or 0 or 1' asXPath evaluate;
		deny: '1 and 1 and 0' asXPath evaluate;
		assert: '1 and 1 and 2' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/5/2015 16:02'!
testNameFunction
	| element |

	(element := XMLElement named: 'test')
		addNode: (XMLElement named: 'test-two');
		addNode: 'test-three' asXMLString.
	self
		assert: (XPath for: 'name()' in: element) = 'test';
		assert: (XPath for: 'name(//.)' in: element) = 'test'.

	element name: 'test' namespaceURI: 'defaultNS'.
	self assert: (XPath for: 'name()' in: element) = 'test'.
	self assert:
		(XPath
			for: 'name()'
			in: element
			context:
				(XPathContext new
					declarePrefix: 'one'
					uri: 'defaultNS')) = 'test'.

	element name: 'prefix:test' namespaceURI: 'prefixNS'.
	self assert: (XPath for: 'name()' in: element) = 'prefix:test'.
	self assert:
		(XPath
			for: 'name()'
			in: element
			context:
				(XPathContext new
					declarePrefix: 'two'
					uri: 'prefixNS')) = 'two:test'.

	self assert: (XPath for: 'name(//text())' in: element) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 04:11'!
testNamespaceAxis
	| nodes |

	nodes := XPathNodeSet new.
	namespacedDocument allElementsDo: [:each |
		nodes
			add: (XPathNamespaceNode withXMLNamespaceOnElement: each);
			add: (XPathNamespaceNode element: each name: '' uri: 'defaultNS');
			add: (XPathNamespaceNode element: each name: 'book' uri: 'bookNS');
			add: (XPathNamespaceNode element: each name: 'ps' uri: 'personNS')].
	self assert: nodes size = (namespacedDocument allElements size * 4).
	self
		assertNodeSet: (namespacedDocument xPath: '//namespace::*')
		hasNodes: nodes.
	self
		assertNodeSet: (namespacedDocument xPath: '//namespace::book')
		hasNodes: (nodes select: [:each | each name ='book']).

	"namespace nodes can't be prefixed or namespaced themselves:"
	self
		assert: (namespacedDocument xPath: '//namespace::book:*') isEmpty;
		assert: (namespacedDocument xPath: '//namespace::book/namespace::book') isEmpty.

	self
		assertNodeSet: (document xPath: '//namespace::*')
		hasNodes:
			(document allElementsCollect: [:each |
				XPathNamespaceNode withXMLNamespaceOnElement: each]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 04:12'!
testNamespaceNameTest
	self
		assertNodeSet: (document xPath: '//namespace::namespace( xml )')
		hasNodes: (document allElements collect: [:each | each namespaceNodes first]).

	self assert: (document xPath: '//namespace::namespace( undeclared )') isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 04:12'!
testNamespaceTypeTest
	| results |

	results := XMLNodeList new.
	namespacedDocument allNodesDo: [:each |
		each isElement
			ifTrue: [results addAll: each namespaceNodes]
			ifFalse: [
				each isStringNode
					ifTrue: [results addLast: each]]].
	self
		assertNodeSet:
			(namespacedDocument xPath: '//namespace::node() | //text()')
		hasNodes: results.
	self
		assertNodeSet:
			(namespacedDocument xPath:
				'(//namespace::node() | //text())/ self :: namespace()')
		hasNodes: (results select: [:each | each isNamespace]).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/13/2015 21:32'!
testNamespaceURIFunction
	self
		assert: (document xPath: 'namespace-uri(/*)') isEmpty;
		assert: (document root xPath: 'namespace-uri()') isEmpty;
		assert: (document xPath: 'namespace-uri(/*/*)') isEmpty;
		assert: (namespacedDocument xPath: 'namespace-uri(/*)') = 'bookNS';
		assert: (namespacedDocument root xPath: 'namespace-uri()') = 'bookNS';
		assert: (namespacedDocument xPath: 'namespace-uri(/*/*)') = 'personNS'! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/13/2015 21:33'!
testNestedFunctionCalls
	self
		assert: 'number(ceiling(3.5) + floor(1.5))' asXPath evaluate = 5;
		assert: 'contains(concat(concat("one", "two"), string("three")), "two")' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/2/2015 04:48'!
testNodeSetAndBooleanComparing
	self assert:
		('<list><a>not-empty</a></list>' parseXML xPath:
			'list/b < true()
				and list/b <= true()
				and list/a > false()
				and list/a >= false()
				and false() < list/a
				and true() <= list/a
				and true() > list/b
				and false() >= list/b
				and not(list/b < false())
				and not(list/a <= false())
				and not(list/b > true())
				and not(list/b >= true())
				and not(true() < list/a)
				and not(true() <= list/b)
				and not(true() > list/a)
				and not(false() >= list/a)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 04:13'!
testNodeSetAndNonNodeSetEquality
	self assert:
		('<list><a>1</a><a>two</a></list>' parseXML xPath:
			'list/a = 1.0
				and 1.0 = list/a
				and not(list/a = 2.0)
				and not(2.0 = list/a)
				and list/a = "two"
				and "two" = list/a
				and not(list/a = "one")
				and not("one" = list/a)
				and list/a = true()
				and true() = list/a
				and not(list/absent = true())
				and not(true() = list/absent)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 04:13'!
testNodeSetAndNonNodeSetInequality
	self assert:
		('<list><a>1</a><b>two</b></list>' parseXML xPath:
			'list/a !!= 2.0
				and 2.0 !!= list/a
				and not(list/a !!= 1.0)
				and not(1.0 !!= list/a)
				and list/b !!= "one"
				and "one" !!= list/b
				and not(list/b !!= "two")
				and not("two" !!= list/b)
				and list/a !!= false()
				and false() !!= list/a
				and list/absent !!= true()
				and true() !!= list/absent')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/2/2015 04:41'!
testNodeSetAndNumberComparing
	self assert:
		('<list><a>1</a><a>2</a></list>' parseXML xPath:
			'list/a < 2
				and list/a <= 1
				and list/a > 1
				and list/a >= 2
				and 1 < list/a
				and 2 <= list/a
				and 2 > list/a
				and 2 >= list/a
				and not(list/a < 1)
				and not(list/a <= 0)
				and not(list/a > 3)
				and not(list/a >= 3)
				and not(3 < list/a)
				and not(3 <= list/a)
				and not(0 > list/a)
				and not(0 >= list/a)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/21/2015 17:22'!
testNodeSetComparing
	self assert:
		('<list>
			<a>1</a><a>2</a><a>3</a>
			<b>0</b>
			<c>4</c>
		</list>' parseXML xPath:
			'list/a < list/a
				and list/a <= list/a
				and list/a > list/a
				and list/a >= list/a
				and list/b < list/a
				and list/b <= list/a
				and not(list/b > list/a)
				and not(list/b >= list/a)
				and not(list/c < list/a)
				and not(list/c <= list/a)
				and list/c > list/a
				and list/c >= list/a')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/21/2015 17:26'!
testNodeSetEquality
	self assert:
		('<list>
			<a>one</a><a>two</a><a>three</a>
			<b>four</b>
		</list>' parseXML xPath:
			'list/a = list/a
				and list/a !!= list/a
				and not(list/b = list/a)
				and list/b !!= list/a
				and not(list/absent = list/a)
				and not(list/absent = list/absent)
				and not(list/absent !!= list/a)
				and not(list/absent !!= list/absent)')! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/13/2015 05:17'!
testNormalizeSpaceFunction
	self assert:
		'normalize-space("
		    one		  
		    two
		    three
		")' asXPath evaluate = 'one two three'.
	self assert: 'normalize-space(" 	")' asXPath evaluate isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/14/2015 16:15'!
testNotFunction
	self	
		deny: 'not(true())' asXPath evaluate;
		assert: 'not(false())' asXPath evaluate;
		deny: 'not(1) or not("1")' asXPath evaluate;
		assert: 'not(0) and not("")' asXPath evaluate;
		deny: (document xPath: 'not(/node())');
		assert: (document xPath: 'not(/text())').
	self
		should: ['not()' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/13/2015 06:02'!
testNumberTokenizing
	self
		assert: ' 1 ' asXPath evaluate = 1;
		assert: ' 12 ' asXPath evaluate = 12;
		assert: ' 1.5 ' asXPath evaluate = 1.5;
		assert: ' 12.5 ' asXPath evaluate = 12.5;
		assert: ' -1 ' asXPath evaluate = -1;
		assert: ' -12 ' asXPath evaluate = -12;
		assert: ' -1.5 ' asXPath evaluate = -1.5;
		assert: ' -12.5 ' asXPath evaluate = -12.5;
		assert: ' .1 ' asXPath evaluate = 0.1;
		assert: ' .12 ' asXPath evaluate = 0.12;
		assert: ' -.1 ' asXPath evaluate = -0.1;
		assert: ' -.12 ' asXPath evaluate = -0.12! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testOneLevel
	| path results |
	path := XPath for: 'addressbook/person'.
	results := path in: document.
	self assert: (results size = 4).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/20/2015 05:35'!
testParentAxis
	self
		assertNodeSet:
			(XPath
				for: '//parent::*'
				in: document)
		hasNodes: (document allElementsSelect: [:each | each hasChildren]).

	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/parent::*'
				in: document)
		hasNodes: (XMLNodeList with: document root).

	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/node()/parent::*'
				in: document)
		hasNodes: (document allElementsNamed: 'person').

	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/node()/parent::*[1][position()=1][last()=1]'
				in: document)
		hasNodes: (document allElementsNamed: 'person').
	self assert:
		(XPath
			for: 'addressbook/person/node()/parent::*[2]'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'addressbook/person/node()/parent::*[position()+1]'
			in: document) isEmpty.

	self
		assertNodeSet:
			(XPath
				for: 'parent::*[1][position()=1][last()=1]'
				in: (document findElementNamed: 'person'))
		hasNodes: (XMLNodeList with: document root).
	self assert:
		(XPath
			for: 'parent::*[2]'
			in: (document findElementNamed: 'person')) isEmpty.
	self assert:
		(XPath
			for: 'parent::*[position() + 1]'
			in: (document findElementNamed: 'person')) isEmpty.

	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/attribute::node()/parent::node()'
				in: document)
		hasNodes: (document allElementsNamed: 'person').

	self assert:
		(XPath
			for: 'parent::node()'
			in: document) isEmpty.
	self assert:
		(XPath
			for: 'addressbook/parent::*'
			in: document) isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/parent::node()'
				in: document)
		hasNodes: (XMLNodeList with: document)! !

!XPathTest methodsFor: 'tests' stamp: 'monty 6/17/2015 02:19'!
testPositionFunction
	self assert:
		(XPath
			for: ' position( ) '
			in: document) = 1.

	(document allElementsNamed: 'person') do: [:each |
		self assert: (XPath for: ' position() ' in: each) = 1].

	(document allElementsNamed: 'person')
		withIndexDo: [:each :i |
			self assert:
				(XPath
					for: 'addressbook/person[position() = ', i printString, ']'
					in: document) = (XMLNodeList with: each).
			self assert:
				(XPath
					for: 'addressbook/person[ ', i printString, ' ]'
					in: document) = (XMLNodeList with: each)].

	self
		should: ['position(0)' asXPath evaluate]
		raise: XPathException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:08'!
testPrecedence
	self
		assert: (XPath for: '1 + 2 * 3 + 4 div 2' in: nil) rounded = 9;
		assert: (XPath for: '(1 + 2) * (3 + 4) mod 5' in: nil) rounded = 1.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 05:35'!
testPrecedingAxis
	| persons preceding |

	persons := document allElementsNamed: 'person'.
	preceding := XMLNodeList new.
	persons allButLast do: [:each | preceding addAll: each allNodes].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/preceding::node()'
				in: document)
		hasNodes: preceding.

	preceding removeAll.
	persons do: [:person |
		 person nodes allButLast do: [:each | preceding addAll: each allNodes]].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person/node()/preceding::node()'
				in: document)
		hasNodes: preceding.

	preceding removeAll.
	persons allButLast do: [:each | preceding addAll: each allNodes].
	preceding addAll: persons last descendentNodes allButLast.
	self
		assertNodeSet:
			(XPath
				for: '//preceding::node()'
				in: document)
		hasNodes: preceding.

	preceding removeAll.
	persons allButLast do: [:person | preceding addAll: person allNodes].
	self
		assertNodeSet:
			(XPath
				for: '(//person[last()]|//person[2]/node()[last()])/preceding::node()'
				in: document)
		hasNodes: preceding.
	self
		assertNodeSet:
			(XPath
				for: '(//person[last()]|//person[1])/preceding::node()'
				in: document)
		hasNodes: preceding.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 05:50'!
testPrecedingSiblingAxis
	| persons siblings |

	persons := document allElementsNamed: 'person'.
	siblings := persons allButLast.
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person[4]/preceding-sibling::node()'
				in: document)
		hasNodes: siblings.

	siblings := (document allElementsNamed: 'manager') allButLast.
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person[4]/preceding-sibling::node()/manager'
				in: document)
		hasNodes: siblings.

	siblings := (document allElementsNamed: 'person') allButLast.
	persons last
		descendentNodesDo: [:each |
			each nextNode
				ifNotNil: [siblings addLast: each]].
	self
		assertNodeSet:
			(XPath
				for: 'addressbook/person[4]//preceding-sibling::node()'
				in: document)
		hasNodes: siblings.

	siblings :=
		document allNodesSelect: [:each |
			each hasParent
				and: [each nextNode notNil]].
	self
		assertNodeSet:
			(XPath for: '//preceding-sibling::node()' in: document)
		hasNodes: siblings.

	siblings := persons copyFrom: 1 to: 3.
	siblings addAll: persons second nodes allButLast.
	self
		assertNodeSet:
			(XPath
				for: '(//person[last()]|//person[2]/node()[last()])/preceding-sibling::node()'
				in: document)
		hasNodes: siblings.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 03:01'!
testPrefixWildcardTest
	| persons |

	persons := namespacedDocument allElementsNamed: 'person'.
	self
		assertNodeSet: (namespacedDocument xPath: '*:addressbook/*:person')
		hasNodes: persons.
	self assert: (namespacedDocument xPath: '*:addressbook/*:contaxt-info') isEmpty.
	self
		assertNodeSet:
			(XPath
				for: 'one:addressbook/*:person'
				in: namespacedDocument
				context: (XPathContext new declarePrefix: 'one' uri: 'bookNS'))
		hasNodes: persons.
	self assert:
		(XPath
			for: 'one:addressbook/*:contact-info'
			in: namespacedDocument
			context: (XPathContext new declarePrefix: 'one' uri: 'bookNS')) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:51'!
testProcessingInstructionTypeTest
	| xml |

	xml := '<root>before<?one two?><?three four?>after</root>' parseXML.
	self
		assertNodeSet: (xml xPath: '// processing-instruction ()')
		hasNodes: (xml root nodes copyFrom: 2 to: 3).
	self
		assertNodeSet: (xml xPath: '// processing-instruction ( "three" )')
		hasNodes: (xml root nodes copyFrom: 3 to: 3).
	self
		assertNodeSet: (xml xPath: '// processing-instruction ( one )')
		hasNodes: (xml root nodes copyFrom: 2 to: 2).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/3/2015 15:29'!
testQualfiedNameTest
	| persons mappings |

	persons := namespacedDocument allElementsNamed: 'person'.
	mappings := 
		Array
			with: 'one' -> 'bookNS'
			with: 'two' -> 'personNS'.
	self
		assert: (namespacedDocument xPath: 'book:addressbook/ps:person') = persons;
		assert: (namespacedDocument xPath: 'book:addressbook/undeclared:person') isEmpty.
	self assert:
		(XPath
			for: 'one:addressbook/two:person'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) = persons.
	self assert:
		(XPath
			for: 'one:addressbook/one:person'
			in: namespacedDocument
			context: (XPathContext namespaces: mappings)) isEmpty.
	self
		should: [
			(XPath
				for: 'undeclared:addressbook'
				in: namespacedDocument
				context: (XPathContext namespaces: mappings))]
		raise: XPathUndeclaredPrefixException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/11/2015 02:49'!
testReverseDocumentOrderPreserving
	| expected result |

	expected := document allNodes last ancestorNodes reversed.
	result :=
		document allNodes last xPath:
			'ancestor-or-self::node()/parent::node()[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected := document allNodes last ancestorNodes reversed.
	result :=
		document allNodes last xPath:
			'ancestor::node()/self::node()[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	document allNodes last ancestorElements reverseDo: [:each |
		expected addAll: each attributeNodes].
	result := document allNodes last xPath: 'ancestor::*/@*[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	document allNodes last ancestorElements reverseDo: [:each |
		expected addAll: each namespaceNodes].
	result := document allNodes last xPath: 'ancestor::*/@@*[true()]'.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:59'!
testReverseForwardAxisCombination
	| nodes |

	nodes := XPathNodeSet new.
	(document allElementsNamed: 'person') allButLast do: [:person |
		person descendentNodesDo: [:each |
			nodes addLast: each]].
	self
		assertNodeSet:
			(XPath for: '*/person[last()]/preceding::node()/node()' in: document)
		hasNodes: nodes.

	nodes := (document allElementsNamed: 'person') allButLast.
	self
		assertNodeSet:
			(XPath
				for: '*/person[last()]/preceding-sibling::node()/self::*[1][position()>0]'
				in: document)
		hasNodes: nodes.
	self
		assert:
			(XPath
				for: '*/person[last()]/preceding-sibling::node()/self::*[1][2]'
				in: document) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/10/2015 09:39'!
testSelfAxis
	self
		assertNodeSet: (XPath for: ' / self::node() ' in: document)
		hasNodes: (XMLNodeList with: document).
	self
		assertNodeSet: (XPath for: ' /*/person/self::* ' in: document)
		hasNodes: (document allElementsNamed: 'person').
	self
		assertNodeSet: (XPath for: ' /*/person/node()/self::node() ' in: document)
		hasNodes:
			(document allNodesSelect: [:each |
				each hasParent
					and: [each parent isElementNamed: 'person']]).

	self
		assertNodeSet:
			(XPath
				for: ' /*/person/self::node()[1][position() = 1][last() = 1] '
				in: document)
		hasNodes: (document allElementsNamed: 'person').
	self
		assert: (XPath for: ' /*/person/self::node()[2] ' in: document) isEmpty;
		assert: (XPath for: ' /*/person/self::node()[position() + 1] ' in: document) isEmpty.

	self
		assertNodeSet:
			(XPath
				for: 'self::node()[1][position() = 1][last() = 1]'
				in: document)
		hasNodes: (XMLNodeList with: document).
	self
		assert: (XPath for: 'self::node()[2]' in: document) isEmpty;
		assert: (XPath for: 'self::node()[position() + 1]' in: document) isEmpty.

	self
		assertNodeSet: (XPath for: ' //person[1]/attribute::node()/. ' in: document)
		hasNodes: (document findElementNamed: 'person') attributeNodes.
	self assert:
		(XPath
			for: ' //person[1]/attribute::node()/self::* '
			in: document) isEmpty.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 6/4/2015 20:22'!
testSorting
	| allNodes |

	allNodes := document allNodes.
	(Array
		with: (XPathNodeSet newFrom: allNodes)
		with: (XPathNodeSet newFrom: allNodes reversed))
		do: [:nodeSet |
			self
				assert: nodeSet sorted = allNodes;
				assert: nodeSet sorted reversed sorted = allNodes].! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 07:17'!
testStartsWithFunction
	self
		assert: 'starts-with("onetwothree", "one")' asXPath evaluate;
		deny: 'starts-with("onetwothree", "ONE")' asXPath evaluate;
		assert: 'starts-with("123", 1)' asXPath evaluate! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 20:08'!
testStringLengthFunction
	self
		assert: 'string-length("test")' asXPath evaluate = 4;
		assert: 'string-length(false())' asXPath evaluate = 5;
		assert: ('string-length()' asXPath in: 'string-value' asXMLString) = 12! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:40'!
testSubstringAfterFunction
	self
		assert: 'substring-after("onetwothree", "one")' asXPath evaluate = 'twothree';
		assert: 'substring-after("onetwothree", ''two'')' asXPath evaluate = 'three';
		assert: 'substring-after("onetwothree", "three")' asXPath evaluate isEmpty;
		assert: 'substring-after("onetwothree", "four")' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:40'!
testSubstringBeforeFunction
	self
		assert: 'substring-before("onetwothree", "one")' asXPath evaluate isEmpty;
		assert: 'substring-before("onetwothree", ''two'')' asXPath evaluate = 'one';
		assert: 'substring-before("onetwothree", "three")' asXPath evaluate = 'onetwo';
		assert: 'substring-before("onetwothree", "four")' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 20:41'!
testSubstringFunction
	self
		assert: 'substring("onetwothree", 4, 3)' asXPath evaluate = 'two';
		assert: 'substring("onetwothree", 4)' asXPath evaluate = 'twothree';
		assert: 'substring("onetwothree", -3, 2)' asXPath evaluate = 'on';
		assert: 'substring("onetwothree", 12, 2)' asXPath evaluate isEmpty;
		assert: 'substring("", 0, 1)' asXPath evaluate isEmpty;
		assert: 'substring("", 0)' asXPath evaluate isEmpty! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/14/2015 14:15'!
testSumFunction
	self assert:
		('<nums><one>1</one><two>2</two><three>3</three></nums>' parseXML
			xPath: 'sum(/nums/*)') = 6.
	self
		should: ['sum()' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:45'!
testTextTypeTest
	self
		assertNodeSet: (document xPath: ' //text( ) ')
		hasNodes: (document allNodesSelect: [:each | each isStringNode]).
	self
		assertNodeSet: (document xPath: '//text()/self::text()')
		hasNodes: (document allNodesSelect: [:each | each isStringNode]).
	self assert: (document xPath: ' //text()/text()') isEmpty.

	self
		should: [document xPath: '//text(name)']
		raise: XPathSyntaxException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testThreeLevel
	| path results |
	path := XPath for: '*/person/contact-info/email'.
	results := path in: document.
	self assert: (results size = 2).! !

!XPathTest methodsFor: 'tests' stamp: 'hfm 5/18/2013 16:58'!
testTopLevel
	| path results elem |
	path := XPath for: 'addressbook'.
	results := path in: document.
	
	self assert: results size = 1.
	
	elem := results first.
	self assert: elem isElement.
	self assert: elem name = 'addressbook'.
	self assert: elem elements size = 4.
	self assert: elem elements first isElement.
	self assert: elem elements first name = 'person'.
	self assert: elem elements last name = 'person'.	! !

!XPathTest methodsFor: 'tests' stamp: 'monty 2/10/2015 06:56'!
testTranslateFunction
	self
		assert: 'translate("onetwothree", "oe", "O")' asXPath evaluate = 'OntwOthr';
		assert: 'translate("onetwothree", "oe", "OEN")' asXPath evaluate = 'OnEtwOthrEE';
		assert: 'translate("onetwothree", "z", "x")' asXPath evaluate = 'onetwothree'! !

!XPathTest methodsFor: 'tests' stamp: 'monty 3/25/2015 03:32'!
testTrueFunction
	self assert: 'true()' asXPath evaluate.
	self
		should: ['true(1)' asXPath evaluate]
		raise: XPathArgumentNumberException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 1/15/2015 04:12'!
testTwoLevel
	| path results |
	path := XPath for: '/*/person/address'.
	results := path in: document.
	self assert: (results size = 4).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/18/2015 21:45'!
testUndeclaredFunctions
	self
		should: [XPath for: 'undeclared()/addressbook' in: document]
		raise: XPathUndeclaredFunctionException.
	self
		should: [XPath for: '(undeclared())/addressbook' in: document]
		raise: XPathUndeclaredFunctionException.
	self
		should: [XPath for: 'addressbook[undeclared()]' in: document]
		raise: XPathUndeclaredFunctionException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 11/18/2015 21:45'!
testUndeclaredVariables
	self
		should: [XPath for: '$undeclared/addressbook' in: document]
		raise: XPathUndeclaredVariableException.
	self
		should: [XPath for: '($undeclared)/addressbook' in: document]
		raise: XPathUndeclaredVariableException.
	self
		should: [XPath for: 'addressbook[$undeclared]' in: document]
		raise: XPathUndeclaredVariableException.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/10/2015 09:55'!
testUnion
	| expected result |

	expected := document allElementsNamed: 'person'.
	result := 
		XPath
			for:
				'addressbook/person[position() = 1 or position() = 4] |
				(addressbook/person[4]/ preceding-sibling::* [position() < 4])'
			in: document.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	document allElementsNamed: 'person' do: [:each |
		expected
			addLast: each;
			addAll: each namespaceNodes;
			addAll: each attributeNodes;
			addAll: each elements].
	result :=
		XPath
			for: 'addressbook/person/@* | */person/@@* | addressbook/person | */person/*'
			in: document.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.

	expected removeAll.
	(document allElementsNamed: 'person') first
		namespaceNodesDo: [:each | expected addLast: each];
		attributeNodesDo: [:each | expected addLast: each];
		nodesDo: [:each | expected addLast: each].
	result :=
		XPath
			for: 'addressbook/person[1]/node() | */person[1]/@* | addressbook/person[1]/@@*'
			in: document.
	self
		assert: result = expected;
		assertNodeSet: result hasSortOrder: XPathDocumentSortOrder.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 4/16/2015 14:38'!
testVariables
	| context result |

	(context := XPathContext new)
		variableAt: 'doc' put: document;
		variableAt: 'pos' ifAbsentPut: [4].
	result :=
		(XPath for: '$doc/addressbook/person[position() !!= $pos]')
			in: nil
			context: context.
	self assert: result = (document allElementsNamed: 'person') allButLast.

	context :=
		XPathContext variables:
			(Dictionary
				with: 'doc' -> document
				with: 'city' -> 'Los Angeles').
	result :=
		(XPath for: '($doc)/addressbook/person[.//address[@city=$city]]')
			in: nil
			context: context.
	self
		assert: result size = 2;
		assert:
			result =
				((document allElementsNamed: 'person') select: [:each |
					((each elementAt: 'address') attributeAt: 'city') = 'Los Angeles']).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:41'!
testWildcard
	| path results |

	path := XPath for: 'addressbook/*'.
	results := path in: document.
	self assert: results size = 4.
	self
		assertNodeSet: results
		hasNodes: (document allElementsNamed: 'person').! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:40'!
testWildcardDescendant
	| path results |

	path := XPath for: '//*'.
	results := path in: document.
	self assert: (results size = 28).
	self
		assertNodeSet: results
		hasNodes: document descendentElements.! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:40'!
testWildcardDescendantAttribute
	| path results |
	path := XPath for: '//*/@city'.
	results := path in: document.
	self assert: results size = 4.
	self
		assertNodeSet: results
		hasNodes:
			((document allElementsSelect: [:each | each includesAttribute: 'city'])
				collect: [:each | each attributeNodeAt: 'city']).! !

!XPathTest methodsFor: 'tests' stamp: 'monty 10/8/2015 02:23'!
testWildcardDescendantAttributeValue
	| path results |
	path := XPath for: ' // * [ @ city = "Los Angeles" ] '.
	results := path in: document.
	self assert: results size = 2.
	self
		assertNodeSet: results
		hasNodes:
			(document allElementsSelect: [:each |
				(each isNamed: 'address')
					and: [(each attributeAt: 'city') = 'Los Angeles']]).! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:39'!
testAncestorAxis
	| nodeWithElements |

	super testAncestorAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list |
		list := each %% 'node()'.
		self
			assert: list = each ancestorNodes;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

		list := each %% '*'.
		self
			assert: list = each ancestorElements;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:39'!
testAncestorOrSelfAxis
	| nodeWithElements |

	super testAncestorOrSelfAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list |
		list := each %%~ 'node()'.
		self
			assert: list = (each ancestorNodes copyWithFirst: each);
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

		list := each %%~ '*'.
		each isElement
			ifTrue: [self assert: list = (each ancestorElements copyWithFirst: each)]
			ifFalse: [self assert: list = each ancestorElements].
		self assertList: list hasSortOrder: XPathReverseDocumentSortOrder].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 10/7/2015 03:51'!
testChildAxis
	| nodeWithElements |

	super testChildAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each |
		each hasChildren
			ifTrue: [| list |
				list := each / 'node()'.
				self
					assert: list = each nodes;
					assertList: list hasSortOrder: XPathDocumentSortOrder.

				list := each / '*'.
				self
					assert: list = each elements;
					assertList: list hasSortOrder: XPathDocumentSortOrder.]
			ifFalse: [
				self
					assert: (each / 'node()') isEmpty;
					assert: (each / '*') isEmpty]].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:40'!
testDescendantAxis
	| nodeWithElements |

	super testDescendantAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each |
		each hasChildren
			ifTrue: [| list |
				list := each // 'node()'.
				self
					assert: list = each descendentNodes;
					assertList: list hasSortOrder: XPathDocumentSortOrder.

				list := each // '*'.
				self
					assert: list = each descendentElements;
					assertList: list hasSortOrder: XPathDocumentSortOrder]
			ifFalse: [
				self
					assert: (each // 'node()') isEmpty;
					assert: (each // '*') isEmpty]].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 10/8/2015 09:20'!
testDescendantOrSelfAxis
	| nodeWithElements |

	super testDescendantOrSelfAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each |
		each hasChildren
			ifTrue: [| list |
				list := each //~ 'node()'.
				self
					assert: list = (each descendentNodes copyWithFirst: each);
					assertList: list hasSortOrder: XPathDocumentSortOrder.

				list := each //~ '*'.
				each isElement
					ifTrue: [self assert: list = (each descendentElements copyWithFirst: each)]
					ifFalse: [self assert: list = each descendentElements].
				self assertList: list hasSortOrder: XPathDocumentSortOrder]
			ifFalse: [
				self assert: (each //~ 'node()') = (XPathNodeSet with: each).
				each isElement
					ifTrue: [self assert: (each //~ '*') = (XPathNodeSet with: each)]
					ifFalse: [self assert: (each //~ '*') isEmpty]]].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:57'!
testFollowingAxis
	| nodeWithElements |

	super testFollowingAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list followingNodes |
		list := each >> 'node()'.
		followingNodes := 
			each hasParent
				ifTrue: [
					each parent descendentNodes copyAfter:
						(each hasChildren
							ifTrue: [each descendentNodes last]
							ifFalse: [each])]
				ifFalse: [XPathNodeSet new].
		self
			assert: list = followingNodes;
			assertList: list hasSortOrder: XPathDocumentSortOrder.

		list := each >> '*'.
		self
			assert: list = followingNodes elements;
			assertList: list hasSortOrder: XPathDocumentSortOrder].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 10/7/2015 03:52'!
testFollowingSiblingAxis
	| nodeWithElements |

	super testFollowingSiblingAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list followingSiblings |
		list := each >>> 'node()'.
		followingSiblings :=
			each hasParent
				ifTrue: [each parent nodes copyAfter: each]
				ifFalse: [XPathNodeSet new].
		self
			assert: list = followingSiblings;
			assertList: list hasSortOrder: XPathDocumentSortOrder.

		list := each >>> '*'.
		self
			assert: list = followingSiblings elements;
			assertList: list hasSortOrder: XPathDocumentSortOrder].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 4/9/2015 04:59'!
testPrecedingAxis
	| nodeWithElements |

	super testPrecedingAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list precedingNodes |
		list := each << 'node()'.
		precedingNodes :=
			each hasParent
				ifTrue: [(each parent descendentNodes copyUpTo: each) reversed]
				ifFalse: [XPathNodeSet new].
		self
			assert: list = precedingNodes;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

		list := each << '*'.
		self
			assert: list = precedingNodes elements;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder].! !

!XMLNodeWithElementsTest methodsFor: '*XPath-Tests' stamp: 'monty 10/7/2015 03:55'!
testPrecedingSiblingAxis
	| nodeWithElements |

	super testPrecedingSiblingAxis.

	nodeWithElements := self newNodeWithElements.
	nodeWithElements allNodesDo: [:each | | list precedingSiblings |
		list := each <<< 'node()'.
		precedingSiblings :=
			each hasParent
				ifTrue: [(each parent nodes copyUpTo: each) reversed]
				ifFalse: [XPathNodeSet new].
		self
			assert: list = precedingSiblings;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder.

		list := each <<< '*'.
		self
			assert: list = precedingSiblings elements;
			assertList: list hasSortOrder: XPathReverseDocumentSortOrder].! !

!XMLCommentTest methodsFor: '*XPath-Tests' stamp: 'monty 3/26/2015 18:15'!
testStringValue
	super testStringValue.

	self assert: (self nodeClass string: 'test') stringValue = 'test'.! !
